using System;
using System.Configuration;
using System.Data;
using System.IO;
using System.Web.Mvc;
using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using SYSCore.aControl;
using SYSCore.Models;
using static SYSCore.aControl.EmailSender;

namespace SYSCore.Controllers
{
    public class BatchController : Controller
    {
        private DBManager odb = new DBManager();
        private String gSQL = "";
        private String mDBName = ConfigurationManager.AppSettings["SYSDBName"];
        private DBOracle oOracledb = new DBOracle();

        private DataService dataService = new DataService();


        public ActionResult ReceivePartInform()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_RECEIVE_INFORM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN_E", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    ReceivePartInform receivePart = new ReceivePartInform(ds.Tables[0], ds.Tables[1]);

                    if(ds.Tables[0].Rows.Count > 0 || ds.Tables[1].Rows.Count > 0)
                    {
                        string ReceivePartInformHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("ReceivePartInform");
                        configure.content = ReceivePartInformHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                    //return View("~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "ReceivePartInform");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }


        public ActionResult ReceiveTempPart()
        {
            try
            {
                DataSet dsHeader = new DataSet();
                DataSet ds2 = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR015_GET_DATA_BY_PIC_EMAIL";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;;

                    dsHeader = oOracledb.SqlQueryDS_CMD(cmd);

                    for (int i = 0; i < dsHeader.Tables[0].Rows.Count; i++)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR015_GET_DATA_BY_PART_EMAIL";
                        cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = dsHeader.Tables[0].Rows[i][0].ToString();
                        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                        ds2 = oOracledb.SqlQueryDS_CMD(cmd);

                        ModelReceiveTempPart receivePart = new ModelReceiveTempPart(ds2.Tables[0]);

                        if (ds2.Tables[0].Rows.Count > 0)
                        {
                            string ReceivePartInformHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/ReceiveTempPart.cshtml", receivePart);

                            //Get email configuration
                            EmailConfigure configure = dataService.GetEmailConfigure("ReceiveTempPart");
                            configure.content = ReceivePartInformHTML;
                            configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));
                            configure.to = new string[] { dsHeader.Tables[0].Rows[i][1].ToString() };
                           // configure.cc = new string[] {  };
                            //Send email with attachments
                            EmailSender sender = new EmailSender();
                            sender.Email(configure);
                        }
                    }                                  

                    return Content("Done");

                    //return View("~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "ReceivePartInform");
                    return Content(ex.Message);
                }

               
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }


        private string RenderViewToString(ControllerContext context, string viewPath, object model = null, bool partial = false)
        {
            // first find the ViewEngine for this view
            ViewEngineResult viewEngineResult = null;
            if (partial)
                viewEngineResult = ViewEngines.Engines.FindPartialView(context, viewPath);
            else
                viewEngineResult = ViewEngines.Engines.FindView(context, viewPath, null);

            if (viewEngineResult == null)
                throw new FileNotFoundException("View cannot be found.");

            // get the view and attach the model to view data
            var view = viewEngineResult.View;
            context.Controller.ViewData.Model = model;

            string result = null;

            using (var sw = new StringWriter())
            {
                var ctx = new ViewContext(context, view, context.Controller.ViewData, context.Controller.TempData, sw);
                view.Render(ctx, sw);
                result = sw.ToString();
            }

            return result;
        }
        public ActionResult PickSuspendEmail()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_PICKING_SUSPEND";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    PickSuspendEmail pickSuspend = new PickSuspendEmail(ds.Tables[0]);

                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        string PickSuspendEmailHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/PickSuspendEmail.cshtml", pickSuspend);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("PickSuspendEmail");
                        configure.content = PickSuspendEmailHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "PickSuspendEmail");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }
    }
}
