﻿using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using SYSCore.Attributes;
using SYSCore.Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/hh")]
    public class HandheldController : ApiController
    {
        public HandheldController() { }
        private DBOracle oOracledb = new DBOracle();
        private String gSQL = "";
        private DataModel DataModel = new DataModel();

        [HttpGet, Route("GetPartFromLocation/{location}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPartFromLocation(string location)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_LO_PARTS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPartFromLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetLocationFromPart/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetLocationFromPart(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_PARTS_LO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLocationFromPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetInvoiceDetail")]
        [HandheldAuthorize]
        public IHttpActionResult GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("GetDetailByItem_R")]
        [HandheldAuthorize]
        public IHttpActionResult GetDetailByItem_R([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_SCAN_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDetailByItem_R");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetDetailByItem_S")]
        [HandheldAuthorize]
        public IHttpActionResult SNPE015_S_GetDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_SCAN_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetInvoicePart")]
        [HandheldAuthorize]
        public IHttpActionResult GetInvoicePart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_INV_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetDetailByItem");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReceived")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveReceived([FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_R_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                oOracledb.SqlQueryReturnValue(cmd);

                if (partList.Count > 0)
                {

                    //Run insert detail
                    foreach (var part in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPE015_R_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                        cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                        cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                        cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                        cmd.Parameters.Add("P_RECEIVE_FLAG", OracleDbType.Varchar2).Value = part["RECEIVE_FLAG"];
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_R_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_MANUAL_RECEIVE", OracleDbType.Varchar2).Value = pData["MANUAL_RECEIVE"] ?? "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                    return Json(new { Result = "Done", DocumentNo = documentNo });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: items are not equally." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: status code." });
                }
                else
                {
                    throw (ex);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SaveShelved/{location}")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveShelved([FromBody] dynamic pData, string location)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> invoiceList = new List<dynamic>(pData.INVOICE_LIST);
                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                //Run insert detail
                foreach (var part in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_S_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = part["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = part["INVOICE_NO"];
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                    cmd.Parameters.Add("P_SHELVE_FLAG", OracleDbType.Varchar2).Value = part["SHELVE_FLAG"];
                    cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = part["LOCATION"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }


                //Run insert invoice
                foreach (var invoice in invoiceList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_S_SAVE_INV_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = invoice["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoice["INVOICE_NO"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_S_SAVE_HANDHELD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new { Result = "Done" });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {

                    return Json(new { Result = "Fail", Message = oOracledb.GetErrorMessage(ex) });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveShelved");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SetRelocationStatus/{location}")]
        [HandheldAuthorize]
        public IHttpActionResult SetRelocationStatus(string location, [FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_HH_SET_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pData.status;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();

            }
            catch (OracleException ex)
            {

                throw (ex);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SetRelocationStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveRelocation")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveRelocation([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_HH_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OLD_LOCATION_NO", OracleDbType.Varchar2).Value = pData.OldLocation;
                cmd.Parameters.Add("P_NEW_LOCATION_NO", OracleDbType.Varchar2).Value = pData.NewLocation;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.RelocationQTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();

            }
            catch (OracleException ex)
            {

                throw (ex);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SetRelocationStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("UpdateToken")]
        [HandheldAuthorize]
        public IHttpActionResult UpdateToken([FromBody] dynamic pToken)
        {

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE015_P_GET_REC_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
            cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            DataTable dtHeader = ds.Tables[0];
            DataTable dtDetail = ds.Tables[1];

            string gGUID = pToken["GUID"];
            SYS_Core.aBiz.ReceiveTokenService receiveToken = new SYS_Core.aBiz.ReceiveTokenService();

            string tokenH = receiveToken.GenerateKeyHeader(gGUID, dtHeader);

            //Update Header
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_H";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
            cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenH;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

            oOracledb.SqlQueryReturnValue(cmd);

            //Update Detail
            foreach (DataRow dataDetail in dtDetail.Rows)
            {
                string tokenD = receiveToken.GenerateKeyDetail(gGUID, dtHeader, dataDetail);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = dataDetail["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = dataDetail["PART_NO"];
                cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenD;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }

            return Ok("Done");
        }

        [HttpGet, Route("AvailableTasks")]
        public IHttpActionResult GetAvailableTasks()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_AVAIL_TASKS";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableTasks");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetStockInfo/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetStockInfo(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_STOCK_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetStockInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCountStockRound")]
        [HandheldAuthorize]
        public IHttpActionResult GetCountStockRound()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_HH_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCountStockRound");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCountStockInfo/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCountStockInfo(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_GET_STOCK_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCountStockInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SaveCountStock")]
        [HandheldAuthorize]
        public IHttpActionResult SaveCountStock([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_COUNT_STOCK_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = pData.roundId;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.partNo;
                cmd.Parameters.Add("P_COUNT_QTY", OracleDbType.Decimal).Value = pData.QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveCountStock");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCycleCountDetail/{headerId}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCycleCountDetail(string headerId)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_HH_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(headerId);
                cmd.Parameters.Add("C_RETURN_PART_LIST", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_COUNT_LIST", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(dataSet));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCycleCountDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetLocation/{locationNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetLocation(string locationNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_GET_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = locationNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("CheckLocationMove")]
        [HandheldAuthorize]
        public IHttpActionResult CheckLocationMove([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_LOCATION_MOVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData["LOCATION_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckLocationMove");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("CheckLocationInto")]
        [HandheldAuthorize]
        public IHttpActionResult CheckLocationIntoNewLocation([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_LOCATION_INTO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData["LOCATION_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckLocationIntoNewLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("ConfirmInitialReLocation")]
        [HandheldAuthorize]
        public IHttpActionResult ConfirmInitialReLocation([FromBody] dynamic pData)
        {
            try
            {
                string relocationMode = pData["mode"].ToString();

                if (relocationMode == "Relocation")
                {
                    dynamic pLocation = pData["location"];
                    InitialRelocation(pLocation);
                } else if (relocationMode == "SwapLocation")
                {
                    dynamic pLocationNew = pData["locationNew"];
                    dynamic pLocationOld = pData["locationOld"];
                    SwapLocation(pLocationOld, pLocationNew);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ConfirmInitialReLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        private void InitialRelocation (dynamic pLocation)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pLocation["LOCATION_NO_OLD"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pLocation["LOCATION_NO_NEW"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pLocation["PART_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = "HANDHELD";
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                if (dataTable.Rows[0]["STATUS"].ToString() != "DONE")
                {
                    throw new Exception(dataTable.Rows[0]["MESSAGE"].ToString());
                }
            } catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "InitialRelocation");
                throw (ex);
            }
        }

        private void SwapLocation(dynamic pLocationOld, dynamic pLocationNew)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_SWAP_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pLocationOld["LOCATION_NO"];
                cmd.Parameters.Add("P_PART_NO_OLD", OracleDbType.Varchar2).Value = pLocationOld["PART_NO"];
                cmd.Parameters.Add("P_LOCATION_NO_NEW", OracleDbType.Varchar2).Value = pLocationNew["LOCATION_NO"];
                cmd.Parameters.Add("P_PART_NO_NEW", OracleDbType.Varchar2).Value = pLocationNew["PART_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = "HANDHELD";
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";

                oOracledb.SqlExecute_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SwapLocation");
                throw (ex);
            }
        }

        #region Picking
        [HttpGet, Route("GetPickListByUser")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickListByUser()
        {
            List<PickGroup> pickGroups = new List<PickGroup>();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_GET_BYUSER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { Result = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { Result = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPickNo/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickNo(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_PICK_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { status = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("RemovePickNo/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult RemovePickNo(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_REMOVE_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RemovePickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveProgressPicking")]
        [HandheldAuthorize]
        public IHttpActionResult SaveProgressPicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                List<dynamic> pickGroups = new List<dynamic>(pData.PICK_GROUPS);

                if (pickGroups.Count > 0)
                {
                    foreach (var group in pickGroups)
                    {

                        var pickHeader = group.pickHeader;
                        List<dynamic> pickDetail = new List<dynamic>(group.pickDetail);

                        foreach (var item in pickDetail)
                        {
                            if (item.QTY != null && Convert.ToInt16(item.QTY.ToString()) > 0)
                            {
                                cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNWR006_SAVE_HH";
                                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickHeader.PICKING_LIST_NO.ToString();
                                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PICK_LOT_DATE.ToString());
                                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Int16).Value = Convert.ToInt16(item.PICK_LOT_SEQ.ToString());
                                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = item.INSTRUCTION_NO.ToString();
                                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO.ToString();
                                cmd.Parameters.Add("P_QTY", OracleDbType.Int16).Value = Convert.ToInt16(item.QTY.ToString());
                                cmd.Parameters.Add("P_IS_UNLOCK", OracleDbType.Varchar2).Value = item.IS_UNLOCK;
                                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                                oOracledb.SqlQueryReturnValue(cmd);
                            }
                        }
                    }

                    return Json(new { Result = "Done" });
                }

                return Json(new { Result = "Fail", Message = "NO_ROW_TO_PROCESS" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveProgressPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SavePickingList/{pickListNo}")]
        [HandheldAuthorize]
        public IHttpActionResult SavePickingList(string pickListNo, [FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> pickList = new List<dynamic>(pData.PICK_LIST);

                if (pickList.Count > 0)
                {

                    //Run insert detail
                    foreach (var item in pickList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR006_SAVE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                        cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item["PICK_LOT_DATE"].ToString());
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = item["CUSTOMER_CODE"].ToString();
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = item["BRANCH_CODE"].ToString();
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = item["ORDER_NO"].ToString();
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = item["ITEM_NO"].ToString();
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = item["SUB_ITEM_NO"].ToString();
                        cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = item["PART_NO"].ToString();
                        cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                        cmd = oOracledb.SqlQueryReturnValue(cmd);

                        if (cmd.Parameters["R_STATUS"].Value.ToString() != "DONE")
                        {
                            string message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                            throw new Exception(message);
                        }

                    }

                    return Json(new { Result = "Done" });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePickingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReportProblemPart")]
        [HandheldAuthorize]
        public IHttpActionResult SaveReportProblemPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_CREATE_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pData.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pData.SUB_ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.MISSING_QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePickingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region Checking
        [HttpGet, Route("GetCheckList/{pickListNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCheckList(string pickListNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_HH_CHECK_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCheckList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveCheckList/{pickListNo}")]
        [HandheldAuthorize]
        public IHttpActionResult SaveCheckList(string pickListNo, [FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> pickList = new List<dynamic>(pData.PICK_LIST);

                if (pickList.Count > 0)
                {

                    //Run insert detail
                    foreach (var item in pickList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR007_SAVE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                        cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item["PICK_LOT_DATE"].ToString());
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = item["CUSTOMER_CODE"].ToString();
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = item["BRANCH_CODE"].ToString();
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = item["ORDER_NO"].ToString();
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = item["ITEM_NO"].ToString();
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = item["SUB_ITEM_NO"].ToString();
                        cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = item["PART_NO"].ToString();
                        cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                        cmd = oOracledb.SqlQueryReturnValue(cmd);

                        if (cmd.Parameters["R_STATUS"].Value.ToString() != "DONE")
                        {
                            string message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                            throw new Exception(message);
                        }

                    }

                    return Json(new { Result = "Done" });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePickingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region Pack

        [HttpGet, Route("GetAvailableCase/{caseGroup}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetAvailableCase(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("R_PRINT_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String printDate = cmd.Parameters["R_PRINT_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, PrintDate = printDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Fail", Message = "UNAVAILABLE_CASE_NO" });
                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Done", Message = "UNCASE_AVAILABLE" });

                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPackInstruction/{pickListNo}/{instructionNo}/{allowExists}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPackInstruction(string pickListNo, string instructionNo, string allowExists)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_HH_PACK_INST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_ALLOW_EXISTS", OracleDbType.Varchar2).Value = allowExists;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                DataTable packedData = ds.Tables[1];

                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData), PackedData = DTFM.convertToList(packedData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPackInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetCaseDetail/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCaseDetail(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];

                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPackInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SavePackedData")]
        [HandheldAuthorize]
        public IHttpActionResult SavePackedData([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                string customerCode = pDataList.customerCode;
                string branchCode = pDataList.branchCode;
                List<dynamic> caseList = new List<dynamic>(pDataList.caseList);
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);


                foreach (var data in caseList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_DELETE_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }

                foreach (var data in packingList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePackedData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        #endregion

        #region Delivery
        [HttpGet, Route("GetDelivery/{invoiceNo}/{caseNo?}")]
        [HandheldAuthorize]
        public IHttpActionResult GetDelivery(string invoiceNo, string caseNo = null)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_DELIVERY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceNo;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveUnsentDelivery")]
        [HandheldAuthorize]
        public IHttpActionResult SaveUnsentDelivery([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_UNSENT_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveUnsentDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveSentDelivery")]
        [HandheldAuthorize]
        public IHttpActionResult SaveSentDelivery([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_SENT_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveSentDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
    }
}
