﻿using SYS_Core.aControl;
using System;
using System.Web.Http;
using System.Web.Http.Cors;
using Oracle.ManagedDataAccess.Client;
using System.Data;
using System.Net.Http;
using System.Net;
using SYS_Core.aBiz;
using System.Threading.Tasks;
using System.Configuration;
using SYS_Core.Controllers;
using System.Collections;
using System.Web;
using System.IO;
using System.Linq;
using SYSCore.Attributes;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/K9")]
    public class K9Controller : ApiController
    {

        private DBOracle oOracledb = new DBOracle();
      
        private DataModel DataModel = new DataModel();

        #region SNM117P

        [HttpGet, Route("SNM117P_GetPDILocation/{locationCode?}")]
        [Authorize]
        public IHttpActionResult SNM117P_GetPDILocation(string locationCode = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM117P_PDI_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM117P_SavePDILocation/{locationCode?}")]
        [Authorize]
        public IHttpActionResult SNM117P_SavePDILocation([FromBody] dynamic pLocation, string locationCode = null)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                if (locationCode == null)
                {
                    cmd.CommandText = "SP_NP_SNM117P_PDI_LOCAT_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_CODE"];
                    cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_DESC"];
                    cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pLocation["VENDOR_CODE_MAPPING"];
                    cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Decimal).Value = pLocation["REASON_CODE_MAPPING"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }
                else
                {
                    cmd.CommandText = "SP_NP_SNM117P_PDI_LOCAT_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                    cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_DESC"];
                    cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pLocation["VENDOR_CODE_MAPPING"];
                    cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Decimal).Value = pLocation["REASON_CODE_MAPPING"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                var result = ex.Message.Split('\n');
                var message = result[0].IndexOf("ORA-20001") == 0 ? result[0].Substring(11, result[0].Length - 11) : "Found error, please contact administrator to solve this issue.";
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, message));
            }

            return Ok();
        }


        [HttpPost, Route("SNM117P_DeletePDILocation/{locationCode}")]
        [Authorize]
        public IHttpActionResult SNM117P_DeletePDILocation(string locationCode)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM117P_PDI_LOCATION_DEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Ok();
        }

        #endregion

        #region SNM118P

        [HttpGet, Route("SNM118P_GetModel/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM118P_GetModel(string endItem = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM118P_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM118P_SaveModel/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM118P_SaveModel([FromBody] dynamic pModel, string endItem = null)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                if (endItem == null)
                {
                    cmd.CommandText = "SP_NP_SNM118P_MODEL_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pModel["MAIN_MODEL"];
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pModel["END_ITEM_18"];
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = pModel["END_ITEM_DESC"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }
                else
                {
                    cmd.CommandText = "SP_NP_SNM118P_MODEL_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pModel["MAIN_MODEL"];
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = pModel["END_ITEM_DESC"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                var result = ex.Message.Split('\n');
                var message = result[0].IndexOf("ORA-20001") == 0 ? result[0].Substring(11, result[0].Length - 11) : "Found error, please contact administrator to solve this issue.";
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, message));
            }

            return Ok();
        }


        [HttpPost, Route("SNM118P_DeleteModel/{endItem}")]
        [Authorize]
        public IHttpActionResult SNM118P_DeleteModel(string endItem)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM118P_MODEL_DEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Ok();
        }

        [HttpPost, Route("SNM118P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM132P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM118P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                dt = SNM118P_ValidateData(gFileLocation);
                //ds.Tables.Add(dt);

                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM118P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable SNM118P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            Hashtable hTable = new Hashtable();
            dt = sys.ReadDataExcel(filelocation);
        
            dt.Columns.Add("END_ITEM_18");
            dt.Columns.Add("DESCRIPTION");
            dt.Columns.Add("ERR_MSG");
            dt.Columns.Add("STATUS");

            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {

                if(!string.IsNullOrEmpty(row["End Item 18 digits"].ToString()))
                {
                    if (hTable.Contains(row["End Item 18 digits"].ToString().Trim()))
                    {
                        row["END_ITEM_18"] = row["End Item 18 digits"].ToString().Trim();
                        row["DESCRIPTION"] = row["Description"].ToString().Trim();
                        row["ERR_MSG"] = "Duplicate End Item 18 digits at Seq " + hTable[row["End Item 18 digits"]];
                        row["STATUS"] = "ERROR";
                    }
                    else
                    {
                        hTable.Add(row["End Item 18 digits"].ToString().Trim(), countrow);
                        row["END_ITEM_18"] = row["End Item 18 digits"].ToString().Trim();
                        row["DESCRIPTION"] = row["Description"].ToString().Trim();
                        

                        if (row["End Item 18 digits"].ToString().Trim().Length > 18)
                        {
                            row["ERR_MSG"] = "End Item 18 digits cannot be greater than 18 digits";
                            row["STATUS"] = "ERROR";
                        }
                        else if (row["End Item 18 digits"].ToString().Trim().Length < 18)
                        {
                            row["ERR_MSG"] = "End Item 18 digits cannot less than 18 digits";
                            row["STATUS"] = "ERROR";
                        }
                        else
                        {
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNM118P_CHECK_DUP";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = row["End Item 18 digits"].ToString().Trim();
                            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                            cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                            string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();


                            row["STATUS"] = R_STATUS ;
                            row["ERR_MSG"] = R_MESSAGE == "null" ? string.Empty : R_MESSAGE;


                        }
                    }
                }

                countrow++;
                
            }

            return dt;
        }

        [HttpPost, Route("SNM118P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM118P_SaveUpload([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM118P_SAVE_TEMP";
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = detail["END_ITEM_18"];
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = detail["DESCRIPTION"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM118P_INSERT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM118P_SaveUpload");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM118P_SaveUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNM119P
        [HttpGet, Route("SNM119P_Search/{assemblyPartNo?}")]
        [Authorize]
        public IHttpActionResult SNM119P_Search(string assemblyPartNo = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = assemblyPartNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            } catch(Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNM119P_GetDetail/{assemblyPartNo}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetDetail(string assemblyPartNo)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = assemblyPartNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM119P_Add")]
        [Authorize]
        public IHttpActionResult SNM119P_Add([FromBody] dynamic pBOMDetail)
        {
            OracleCommand cmd;
            try
            {
                int iCount = 0;
                foreach (var bomDetail in pBOMDetail.Detail)
                {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                        cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                        cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_QTY;
                        cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                        cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                        cmd.Parameters.Add("P_SUPER_SESSION_ON", OracleDbType.Varchar2).Value = bomDetail.SUPER_SESSION_ON;
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = bomDetail.FLAG;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                      oOracledb.SqlQueryReturnValue(cmd);
                }

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_BOM_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pBOMDetail.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = pBOMDetail.ASSEMBLY_PART_NO;
                    cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = pBOMDetail.DESCRIPTION;
                    cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = pBOMDetail.SPPS_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                    string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                    if (R_STATUS == "FAIL")
                    {
                        oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Add");
                    }

                    return Json(new { STATUS_MSG = R_STATUS, });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Add");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


      

        [HttpGet, Route("SNM119P_Delete/{installpart}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM119P_Delete(string installpart)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installpart;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM119P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM119P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var bomDetail in param.Detail)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_QTY;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                    cmd.Parameters.Add("P_SUPER_SESSION_ON", OracleDbType.Varchar2).Value = bomDetail.SUPER_SESSION_ON;
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = bomDetail.FLAG;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = param.INSTALL_PART_NO;
                cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = param.SPPS_NO;
                cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = param.DESCRIPTION;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpGet, Route("SNM119P_GetInstallPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM121P_Family_Code(string part, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_INSTALL_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetMaterialPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetMaterialPart(string part, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_MATERIAL_P";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetAssemblyPart/{part}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetAssemblyPart(string part)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_ASSEM_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetSppsCode/{spps}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM130P_Spps_Code(string spps, int noofshowrec)
        {
            DataSet ds = new DataSet();

            spps = ismoUtil.escapeCharacter(spps);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_SPPS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = spps.ToUpper() == "NULL" ? (object)DBNull.Value : spps;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM119P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM119P_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM119P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                SystemApiController sys = new SystemApiController();
                //dt = sys.ReadDataExcel(gFileLocation);
                dt = SNM119P_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable SNM119P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            Hashtable hTableH = new Hashtable();
            Hashtable hTableD = new Hashtable();
            dt.Columns.Add("ERROR_MSG");
            dt.Columns.Add("ASSEMBLY_PART_NO");

            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["INSTALL_PART_NO"].ToString()))
                {

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_VALIDATE_PART";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = row["INSTALL_PART_NO"];
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = row["MATERIAL_PART_NO"];
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = row["MATERIAL_QTY"];
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = row["EXT_COLOR"];
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = row["INT_COLOR"];
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    if (cmd.Parameters["C_RETURN_MSG"].Value.ToString() == "null")
                    {
                        row["ERROR_MSG"] = DBNull.Value;
                    }
                    else
                    {
                        row["ERROR_MSG"] = cmd.Parameters["C_RETURN_MSG"].Value.ToString();
                    }

                    row["ASSEMBLY_PART_NO"] = row["INSTALL_PART_NO"].ToString().Replace(row["INSTALL_PART_NO"].ToString().Substring(1, 1), "8");


                }
                else
                {
                    row["ERROR_MSG"] = "Install Part No not found";
                }


            }

            return dt;
        }


        [HttpPost, Route("SNM119P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM119P_SaveUpload([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var bomDetail in param)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP_UPD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = bomDetail.SPPS_NO;
                    cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = bomDetail.DESCRIPTION;
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Int16).Value = bomDetail.MATERIAL_QTY;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                    cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = bomDetail.ASSEMBLY_PART_NO;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_INSERT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNM120P

        [HttpGet, Route("SNM120P_Search/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM120P_Search(string endItem = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNM120P_GetDetail/{bomKey}")]
        [Authorize]
        public IHttpActionResult SNM120P_GetDetail(string bomKey)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM120P_GetBOMList/{partno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM120P_GetBOMList(string partno, int noofshowrec, [FromBody] dynamic pColor)
        {
            DataSet ds = new DataSet();

            partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM120P_BOM_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = (partno != null && partno.ToUpper() == "NULL" ? "" : partno);
            cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pColor["EXT_COLOR"];
            cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pColor["INT_COLOR"];
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM120P_Add")]
        [Authorize]
        public IHttpActionResult SNM120P_Add([FromBody] dynamic pInstallDetail)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_CHECK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pInstallDetail.END_ITEM_18;
                cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.EXT_COLOR;
                cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.INT_COLOR;
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.PDI_LOCATION_CODE;
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.DESTINATION_CODE;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                if (cmd.Parameters["R_STATUS"].Value.ToString() == "Exist")
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Unable to add existing Install parts."));
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM120P_INSTALL_H_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pInstallDetail.END_ITEM_18;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.INT_COLOR;
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.PDI_LOCATION_CODE;
                    cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.DESTINATION_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_BOM_KEY", OracleDbType.Varchar2, 35).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string BOM_KEY = cmd.Parameters["R_BOM_KEY"].Value.ToString();

                    foreach (var installParts in pInstallDetail.INSTALL_PARTS)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_ADD";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = BOM_KEY;
                        cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installParts.INSTALL_PART_NO;
                        cmd.Parameters.Add("P_INSTALL_QTY", OracleDbType.Decimal).Value = installParts.INSTALL_QTY;
                        cmd.Parameters.Add("P_TAKE_FROM", OracleDbType.Varchar2).Value = installParts.TAKE_FROM;
                        cmd.Parameters.Add("P_ISSUE_DOC_TYPE", OracleDbType.Varchar2).Value = installParts.ISSUE_DOC_TYPE;
                        cmd.Parameters.Add("P_CUSTOMER_TYPE", OracleDbType.Varchar2).Value = installParts.CUSTOMER_TYPE;
                        cmd.Parameters.Add("P_FIX_CUSTOMER_CODE", OracleDbType.Varchar2).Value = installParts.FIX_CUSTOMER_CODE;
                        cmd.Parameters.Add("P_FIX_BRANCH_CODE", OracleDbType.Varchar2).Value = installParts.FIX_BRANCH_CODE;
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                }
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM120P_Save/{bomKey}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM120P_Save([FromBody] dynamic pInstallDetail, string bomKey)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                foreach (var installParts in pInstallDetail.INSTALL_PARTS)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installParts.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_INSTALL_QTY", OracleDbType.Decimal).Value = installParts.INSTALL_QTY;
                    cmd.Parameters.Add("P_TAKE_FROM", OracleDbType.Varchar2).Value = installParts.TAKE_FROM;
                    cmd.Parameters.Add("P_ISSUE_DOC_TYPE", OracleDbType.Varchar2).Value = installParts.ISSUE_DOC_TYPE;
                    cmd.Parameters.Add("P_CUSTOMER_TYPE", OracleDbType.Varchar2).Value = installParts.CUSTOMER_TYPE;
                    cmd.Parameters.Add("P_FIX_CUSTOMER_CODE", OracleDbType.Varchar2).Value = installParts.FIX_CUSTOMER_CODE;
                    cmd.Parameters.Add("P_FIX_BRANCH_CODE", OracleDbType.Varchar2).Value = installParts.FIX_BRANCH_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_H_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM120P_Delete/{bomKey}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM120P_Delete(string bomKey)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }
        #endregion
    }
}
