﻿using SYS_Core.aControl;
using System;
using System.Web.Http;
using System.Web.Http.Cors;
using Oracle.ManagedDataAccess.Client;
using System.Data;
using System.Net.Http;
using System.Net;
using SYS_Core.aBiz;
using System.Threading.Tasks;
using System.Configuration;
using SYS_Core.Controllers;
using System.Collections;
using System.Web;
using System.IO;
using System.Linq;
using SYSCore.Attributes;
using Newtonsoft.Json;
using System.Collections.Generic;
using System.Security.Claims;
using SYS_Core.Services;
using System.Data.SqlClient;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/Report")]
    public class ReportController : ApiController
    {

        private DBManager odb = new DBManager();
        private String gSQL = "";
        private String mDBName = ConfigurationManager.AppSettings["SYSDBName"];

        private DBOracle oOracledb = new DBOracle();
        private DataService dataService = new DataService();
        public TokenService _tokenService = new TokenService();
        #region Report

        [HttpGet, Route("GetPOClass")]
        [ReportAuthorize]
        public HttpResponseMessage GetPOClass()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd = new OracleCommand();
          
            try
            {

                cmd.CommandText = "SP_NP_MT_PO_CLASS_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Request.CreateResponse(HttpStatusCode.OK, dt);

            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.NoContent, dt);
            }

        }

        [HttpPost, Route("GetPONo")]
        [ReportAuthorize]
        public HttpResponseMessage GetPONo([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
           

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_PO_NO_BYID";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param.PO_NO;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            dt = ds.Tables[0];
            return Request.CreateResponse(HttpStatusCode.OK, dt);
        }

        [HttpPost, Route("GetManualPart")]
        [ReportAuthorize]
        public HttpResponseMessage GetManualPart([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            //partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_PRINT_LABEL_MANUALPRINT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 1000;
            cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            dt = ds.Tables[0];
            return Request.CreateResponse(HttpStatusCode.OK, dt);
        }

        [HttpGet, Route("GetIndentDetailPartNo/{partNo}")]
        [ReportAuthorize]
        public HttpResponseMessage GetIndentDetailPartNo(string partNo)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            //partNo = ismoUtil.escapeCharacter(partNo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ088_OrderPartNo";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            dt = ds.Tables[0];
            return Request.CreateResponse(HttpStatusCode.OK, dt);
        }


        [HttpGet, Route("GetVendorCode/{vendorcode}")]
        [ReportAuthorize]
        public HttpResponseMessage GetVendorCode(string vendorcode)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "sp_NP_MT_VENDOR_CODE_ByID";
            cmd.Parameters.Add("p_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorcode ;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            dt = ds.Tables[0];
            return Request.CreateResponse(HttpStatusCode.OK, dt);
        }


        [HttpGet, Route("GetDataPrintLabel")]
        [ReportAuthorize]
        public HttpResponseMessage GetDataPrintLabel()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ088_PRN_RPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Request.CreateResponse(HttpStatusCode.OK, dt);
            }


            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDataPrintLabel");
                return Request.CreateResponse(HttpStatusCode.InternalServerError, "FAIL");
            }
        }

        [HttpGet, Route("GetDataPrintLabelManual")]
        [ReportAuthorize]
        public HttpResponseMessage GetDataPrintLabelManual()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ088_PRN_RPT_NEW";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME); ;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Request.CreateResponse(HttpStatusCode.OK, dt);
            }


            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDataPrintLabelManual");
                return Request.CreateResponse(HttpStatusCode.InternalServerError, "FAIL");
            }
        }

        [HttpGet, Route("GetVendor")]
        [ReportAuthorize]
        public HttpResponseMessage GetVendor()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_MT_VENDOR_CODE";
                //cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME); ;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Request.CreateResponse(HttpStatusCode.OK, dt);
            }


            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDataPrintLabel");
                return Request.CreateResponse(HttpStatusCode.InternalServerError, "FAIL");
            }
        }


        [HttpPost, Route("GenReportByCriteria")]
        [ReportAuthorize]
        public HttpResponseMessage GenReportByCriteria([FromBody] dynamic SRM)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            gSQL = "SP_NP_GET_PO_LABEL";
            try
            {
                
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = SRM.DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(SRM.DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = SRM.DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(SRM.DATE_TO);
                cmd.Parameters.Add("P_FROM_PO_NO", OracleDbType.Varchar2).Value = SRM.FROM_PO == null ? (object)DBNull.Value : SRM.FROM_PO;
                cmd.Parameters.Add("P_TO_PO_NO", OracleDbType.Varchar2).Value = SRM.TO_PO == null ? (object)DBNull.Value : SRM.TO_PO; ;
                cmd.Parameters.Add("P_PO_CLASS", OracleDbType.Varchar2).Value = SRM.PO_CLASS == null ? (object)DBNull.Value : SRM.PO_CLASS; ;
                cmd.Parameters.Add("P_FROM_VENDOR_CODE", OracleDbType.Varchar2).Value = SRM.FROM_VENDOR == null ? (object)DBNull.Value : SRM.FROM_VENDOR; ;
                cmd.Parameters.Add("P_TO_VENDOR_CODE", OracleDbType.Varchar2).Value = SRM.TO_VENDOR == null ? (object)DBNull.Value : SRM.TO_VENDOR; ;
                cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = SRM.VENDOR_TYPE == null ? (object)DBNull.Value : SRM.VENDOR_TYPE; ;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = SRM.ORDER_PART_NO == null ? (object)DBNull.Value : SRM.ORDER_PART_NO; ;
                cmd.Parameters.Add("P_PRINT_STATUS", OracleDbType.Varchar2).Value = SRM.PRINT_STATUS == null ? (object)DBNull.Value : SRM.PRINT_STATUS; ;
                cmd.Parameters.Add("P_IC_INCHARGE", OracleDbType.Varchar2).Value = SRM.IC_INCHARGE == null ? (object)DBNull.Value : SRM.IC_INCHARGE; ;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();

                return Request.CreateResponse(HttpStatusCode.OK, "DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GenReportByCriteria");
                return Request.CreateResponse(HttpStatusCode.OK, "FAIL");

                //System.Console.WriteLine("Exception: {0}", ex.ToString());
            }
        }

        [HttpPost, Route("SavePrintManual")]
        [Authorize]
        public HttpResponseMessage SavePrintManual([FromBody] dynamic manualPrint)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            gSQL = "SP_NP_SNSQ088_PRINT_MANUAL_NEW";

            try
            {
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = manualPrint.PART_NO == null ? (object)DBNull.Value : manualPrint.PART_NO.ToString();
                cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = manualPrint.PART_NAME == null ? (object)DBNull.Value : manualPrint.PART_NAME.ToString();
                cmd.Parameters.Add("P_ORIGIN_NAME", OracleDbType.Varchar2).Value = manualPrint.ORIGIN_NAME == null ? (object)DBNull.Value : manualPrint.ORIGIN_NAME.ToString();
                cmd.Parameters.Add("P_LABEL_QTY", OracleDbType.Varchar2).Value = manualPrint.LABEL_QTY;
                cmd.Parameters.Add("P_LABEL_VALUE", OracleDbType.Varchar2).Value = manualPrint.LABEL_VALUE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Request.CreateResponse(HttpStatusCode.OK, "DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePrintManual");
            }

            return Request.CreateResponse(HttpStatusCode.InternalServerError, "FAIL");
        }

        [HttpPost, Route("GetReceivingCard")]
        [ReportAuthorize]
        public HttpResponseMessage GetReceivingCard([FromBody] dynamic SRM)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd = new OracleCommand();
            
            try
            {

                cmd.CommandText = "SP_NP_RECEIVING_CARD_RPT";
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = SRM.VENDOR_CODE == null ? (object)DBNull.Value : SRM.VENDOR_CODE;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = SRM.INVOICE_NO == null ? (object)DBNull.Value : SRM.INVOICE_NO;
                cmd.Parameters.Add("P_RECEIVING_CARD_NO", OracleDbType.Varchar2).Value = SRM.RECEIVING_CARD_NO == null ? (object)DBNull.Value : SRM.RECEIVING_CARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Request.CreateResponse(HttpStatusCode.OK, dt);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetReceivingCard");
                return Request.CreateResponse(HttpStatusCode.OK, "FAIL");

                //System.Console.WriteLine("Exception: {0}", ex.ToString());
            }
        }
        [HttpPost, Route("Login")]
        public async Task<HttpResponseMessage> Login([FromBody] dynamic loginObj)
        {
            try
            {
                DataTable dt = new DataTable();
                DataTable ssdt = new DataTable();
                string gUsername = loginObj.username.ToString();
                string gPassword = loginObj.password.ToString();
                string gSystem = "01";//loginObj.company.ToString();
                DataSet ds = new DataSet();

                gSQL = "EXEC [sp_User_ByUserName] '{0}'";
                gSQL = String.Format(gSQL, gUsername);
                ssdt = odb.SqlQuery(gSQL, mDBName);

                if (ssdt.Rows[0]["RESULT"].ToString() != "DONE")
                {
                    var data = new
                    {
                        token = "Incorrect username or password.",
                        role = ""
                    };
                    return Request.CreateResponse(HttpStatusCode.NotAcceptable, data);
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Login";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pPASSWORD", OracleDbType.Varchar2, gPassword, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("pRETURN_MSG", OracleDbType.Varchar2, ParameterDirection.Output);
                cmd.Parameters["pRETURN_MSG"].Size = 200;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string gMsg = cmd.Parameters["pRETURN_MSG"].Value.ToString();

                if (gMsg != "DONE")
                {
                    // if found error in database return
                    var data = new
                    {
                        token = gMsg,
                        role = ""
                    };
                    return Request.CreateResponse(HttpStatusCode.NotAcceptable, data);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Info";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];

                var Result = dt.Rows[0];

                if (ds.Tables.Count > 0)
                {
                    var claims = new List<Claim>
                    {
                    new Claim(Cons.SS_COMPANY_CODE, gSystem),
                    new Claim(Cons.SS_USER_ID, ssdt.Rows[0]["USER_ID"].ToString()),
                    new Claim(Cons.SS_USER_NAME, Result["SK_USER_ID"].ToString()),
                    new Claim(Cons.SS_USER_NAME_TH, Result["TNAME"].ToString()),
                    new Claim(Cons.SS_USER_NAME_EN, Result["ENAME"].ToString()),
                    new Claim(Cons.SS_USER_LANG, Result["LANGUAGE"].ToString()),
                    new Claim(Cons.SS_USER_DEPT, Result["SK_GRP_USER_ID"].ToString()),

                    new Claim(Cons.SS_USER_ROLE, ssdt.Rows[0]["ROLE_ID"].ToString()),
                    new Claim(Cons.SS_IP_ADDRESS, HttpContext.Current.Request.UserHostAddress.ToString())
                    };

                    var jwtToken = _tokenService.GenerateAccessTokenHandheld(claims);

                    var data = new
                    {
                        token = jwtToken,
                        role = Result["SK_GRP_USER_ID"].ToString()
                    };
                    
                    return Request.CreateResponse(HttpStatusCode.OK, data);

                }
                var data1 = new
                {
                    token = "Incorrect username or password.",
                    role = ""
                };
                return Request.CreateResponse(HttpStatusCode.NotAcceptable, data1);

            }
            catch (Exception ex)
            {
                var data1 = new
                {
                    token = "Incorrect username or password.",
                    role = ""
                };
                return Request.CreateResponse(HttpStatusCode.NotAcceptable, data1);
            }
        }


        [HttpGet, Route("GetReportViewerList")]
        [ReportAuthorize]
        public HttpResponseMessage GetReportViewerList()
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_List";
                command.Parameters.Add("@ROLE_ID", SqlDbType.Int).Value = Convert.ToInt32(DataService.GetIdentity(User, Cons.SS_USER_ROLE));

                DataSet ds = odb.SqlQueryDS(command, mDBName);
                DataTable reportList = ds.Tables[0];

                return Request.CreateResponse(HttpStatusCode.OK, reportList);

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "GetReportViewerList");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message);
            }
        }

        [HttpPost, Route("GetReportViewer")]
        [ReportAuthorize]
        public HttpResponseMessage GetReportViewer([FromBody] dynamic param)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_ByID";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = param.ReportID;
                command.Parameters.Add("@P_USER_ID", SqlDbType.VarChar).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                DataSet ds = odb.SqlQueryDS(command, mDBName);
                return Request.CreateResponse(HttpStatusCode.OK, ds);
            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", "Report ID: " + param.reportId + " " + ex.Message, "GetReportViewer");
                return Request.CreateResponse(HttpStatusCode.ExpectationFailed);
            }
        }
        [HttpPost, Route("RunReportViewer")]
        [ReportAuthorize]
        public IHttpActionResult RunReportViewer([FromBody] dynamic pData)
        {
            try
            {
                string query = String.Concat("SELECT Q.* FROM (", pData.query, ") Q");
                OracleCommand command = new OracleCommand();
                command.CommandType = CommandType.Text;
                command.CommandText = query;
                DataTable dataTable = oOracledb.SqlQueryText(command);
                return Ok(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", "Report ID: " + pData.ID + " " + ex.Message, "GetReportViewer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }
        [HttpPost, Route("SaveReportViewerParams")]
        [ReportAuthorize]
        public IHttpActionResult SaveReportViewerParams([FromBody] dynamic pData)
        {
            try
            {
                var paramList = new List<dynamic>(pData.paramList);
                int count = 0;
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_Clear_Params";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = pData.reportId;
                odb.SqlExecute(command, mDBName);
                command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_Save_Name";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = pData.reportId;
                command.Parameters.Add("@REPORT_NAME", SqlDbType.NVarChar, 500).Value = pData.reportName;
                odb.SqlExecute(command, mDBName);
                foreach (var param in paramList)
                {
                    count++;
                    command = new SqlCommand();
                    command.CommandType = CommandType.StoredProcedure;
                    command.CommandText = "dbo.sp_Report_Viewer_Save_Params";
                    command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = pData.reportId;
                    command.Parameters.Add("@PARAM_NO", SqlDbType.Int).Value = count;
                    command.Parameters.Add("@CONDITION", SqlDbType.NVarChar, 500).Value = param.CONDITION;
                    command.Parameters.Add("@REMARKS", SqlDbType.NVarChar, 500).Value = param.REMARKS;
                    command.Parameters.Add("@IS_ACTIVE", SqlDbType.NVarChar, 10).Value = param.IS_ACTIVE;
                    odb.SqlExecute(command, mDBName);
                }
                return Ok("Done");
            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "SaveReportViewerParams");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }
        #endregion

    }
}
