using SYS_Core.aControl;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Claims;
using System.Web.Http;
using System.Web.Http.Cors;
using SYS_Core.aBiz;
using System.Threading.Tasks;
using System.IO;
using Newtonsoft.Json;
using System.Web;
using System.Net.Http.Headers;
using Oracle.ManagedDataAccess.Client;
//using SYSCore.DataAccess;
using SYS_Core.Services;
using System.IO.Compression;
using Microsoft.Ajax.Utilities;
using ExcelDataReader;
using System.Data.OleDb;
using System.Globalization;
using static SYSCore.aControl.EmailSender;
using SYSCore.aControl;
using System.Data.SqlClient;
using SYSCore.Attributes;

namespace SYS_Core.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/System")]
    public class SystemApiController : ApiController
    {
        private DBManager odb = new DBManager();
        private String gSQL = "";
        private String mDBName = ConfigurationManager.AppSettings["SYSDBName"];

        private DBOracle oOracledb = new DBOracle();
        private DataService dataService = new DataService();

        public TokenService _tokenService = new TokenService();
        //private readonly ITokenService _tokenService;

      

        #region auth
        [HttpPost, Route("PreLogin")]
        public IHttpActionResult PreLogin([FromBody] dynamic loginObj)
        {

            string userName = loginObj.username.ToString();

            if (DateTime.Now.Hour >= 20)
            {
                aControl.DBOracle oOracle = new aControl.DBOracle();
                if (oOracle.CheckBatchRunning())
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.ServiceUnavailable, "Batch is running"));
                }
            }

            return Ok();
        }

        [HttpPost, Route("Login")]
        public async Task<IHttpActionResult> Login([FromBody] dynamic loginObj)
        {
            try
            { 
                DataTable dt = new DataTable();
                DataTable ssdt = new DataTable();
                string gUsername = loginObj.username.ToString();
                string gPassword = loginObj.password.ToString();
                string gSystem = loginObj.company.ToString();
                DataSet ds = new DataSet();

                gSQL = "EXEC [sp_User_ByUserName] '{0}'";
                gSQL = String.Format(gSQL, gUsername);
                ssdt = odb.SqlQuery(gSQL, mDBName);

                if (ssdt.Rows[0]["RESULT"].ToString() != "DONE")
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Incorrect username or password."));
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Login";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pPASSWORD", OracleDbType.Varchar2, gPassword, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("pRETURN_MSG", OracleDbType.Varchar2, ParameterDirection.Output);
                cmd.Parameters["pRETURN_MSG"].Size = 200;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string gMsg = cmd.Parameters["pRETURN_MSG"].Value.ToString();

                if (gMsg != "DONE")
                {
                    // if found error in database return
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, gMsg));
                }

                //--- Get user info after pass log in ----
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Info";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];

                var Result = dt.Rows[0];

                if (ds.Tables.Count > 0)
                {
                    var claims = new List<Claim>
                    {
                        new Claim(Cons.SS_COMPANY_CODE, gSystem),
                        new Claim(Cons.SS_USER_ID, ssdt.Rows[0]["USER_ID"].ToString()),
                        new Claim(Cons.SS_USER_NAME, Result["SK_USER_ID"].ToString()),
                        new Claim(Cons.SS_USER_NAME_TH, Result["TNAME"].ToString()),
                        new Claim(Cons.SS_USER_NAME_EN, Result["ENAME"].ToString()),
                        new Claim(Cons.SS_USER_LANG, Result["LANGUAGE"].ToString()),
                        new Claim(Cons.SS_USER_DEPT, Result["SK_GRP_USER_ID"].ToString()),

                        new Claim(Cons.SS_USER_ROLE, ssdt.Rows[0]["ROLE_ID"].ToString()),
                        new Claim(Cons.SS_IP_ADDRESS, HttpContext.Current.Request.UserHostAddress.ToString())
                    };

                    //var user = _usersDb.Users.SingleOrDefault(u => u.Username == gUsername);

                    var jwtToken = _tokenService.GenerateAccessToken(claims);
                    var refreshToken = _tokenService.GenerateRefreshToken();

                    string TokenRegisterResult = odb.RegisterToken(gUsername, null, null, jwtToken, refreshToken);

                    if (TokenRegisterResult == "VALID")
                    {
                        return Ok(new
                        {
                            token = jwtToken,
                            refreshToken = refreshToken
                        });
                    }
                    else
                    {
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Invalid identifier, token will not be generated"));
                    }
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Incorrect username or password."));
            }
            catch(Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "Found error, please contact adimistrator."));
            }
        }


        [HttpPost, Route("LoginHandheld")]
        public async Task<IHttpActionResult> LoginHandheld([FromBody] dynamic loginObj)
        {

            try
            {

                DataTable dt = new DataTable();
                DataTable ssdt = new DataTable();
                string gUsername = loginObj.username.ToString();
                string gPassword = loginObj.password.ToString();
                string gSystem = "01";
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Login";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pPASSWORD", OracleDbType.Varchar2, gPassword, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("pRETURN_MSG", OracleDbType.Varchar2, ParameterDirection.Output);
                cmd.Parameters["pRETURN_MSG"].Size = 200;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string gMsg = cmd.Parameters["pRETURN_MSG"].Value.ToString();

                if (gMsg != "DONE")
                {
                    // if found error in database return
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, gMsg));
                }

                //--- Get user info after pass log in ----
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SYS_User_Info";
                cmd.Parameters.Add("pUSER_ID", OracleDbType.Varchar2, gUsername, ParameterDirection.Input);
                cmd.Parameters.Add("pSYSTEM_ID", OracleDbType.Varchar2, gSystem, ParameterDirection.Input);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];

                var Result = dt.Rows[0];

                if (ds.Tables.Count > 0)
                {
                    var claims = new List<Claim>
                    {
                    new Claim(Cons.SS_COMPANY_CODE, gSystem),
                    new Claim(Cons.SS_USER_ID, "0"),
                    new Claim(Cons.SS_USER_NAME, Result["SK_USER_ID"].ToString()),
                    new Claim(Cons.SS_USER_NAME_TH, Result["TNAME"].ToString()),
                    new Claim(Cons.SS_USER_NAME_EN, Result["ENAME"].ToString()),
                    new Claim(Cons.SS_USER_LANG, Result["LANGUAGE"].ToString()),
                    new Claim(Cons.SS_USER_DEPT, Result["SK_GRP_USER_ID"].ToString()),

                    new Claim(Cons.SS_USER_ROLE, Result["SK_GRP_USER_ID"].ToString()),
                    new Claim(Cons.SS_IP_ADDRESS, HttpContext.Current.Request.UserHostAddress.ToString())
                    };

                    var jwtToken = _tokenService.GenerateAccessTokenHandheld(claims);

                    return Ok(new
                    {
                        token = jwtToken
                    });

                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "INCORRECT_USER_PWD"));
            }
            catch(Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "LoginHandheld");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "FOUND_ERROR"));
            }
        }

        [HttpPost, Route("GenerateApiToken/{pKey}")]
        public async Task<IHttpActionResult> GenerateApiToken([FromBody] dynamic tokenObj, string pKey)
        {
            string apiName = "";
            try
            {
                if(pKey != ConfigurationManager.AppSettings["JWT_KEY"])
                {
                    throw new Exception("Incorrect key for generate API token.");
                }

                apiName = tokenObj.APIName.ToString();
                string gUsername = "LL_" + apiName;

                var claims = new List<Claim>
                {
                    new Claim(Cons.SS_COMPANY_CODE, "01"),
                    new Claim(Cons.SS_USER_ID, "0"),
                    new Claim(Cons.SS_USER_NAME, gUsername),
                    new Claim("GenerateDate", Convert.ToString(DateTime.Now)),
                    new Claim(Cons.SS_IP_ADDRESS, HttpContext.Current.Request.UserHostAddress.ToString())
                };

                var jwtToken = _tokenService.GenerateAccessTokenNoExpiration(claims);
                string registerStatus = odb.RegisterToken(gUsername, null, null, jwtToken, null, 0);
                if(registerStatus == "VALID")
                {
                    oOracledb.WriteLog("APIToken", "Token generated for API: " + apiName, "GenerateAPIToken");

                    return Ok(new
                    {
                        token = jwtToken
                    });
                } else
                {
                    throw new Exception ("Unable to register token.");
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "API: " + apiName + " --- " + ex.Message, "GenerateAPIToken");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "Found error, please contact adimistrator."));
            }
        }

        [HttpGet, Route("GetUserRoles")]
        [Authorize]
        public IHttpActionResult GetUserRoles()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_User_Roles] '{0}'";
            gSQL = String.Format(gSQL, DataService.GetIdentity(User, Cons.SS_USER_ID));
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        //[HttpPatch, Route("SetRoleID/{roleId}")]
        //[Authorize]
        //public IHttpActionResult SetRoleID(string roleId)
        //{
        //    var identity = User.Identity as ClaimsIdentity;
        //    var roleClaims = new List<Claim>
        //            {
        //              new Claim(Cons.SS_USER_ROLE, roleId),
        //            };

        //    var claims = identity.Claims.Concat(roleClaims).ToList();

        //    string tokenString = ismoUtil.generateToken(claims);

        //    return Ok(tokenString);
        //}

        #endregion

        #region changePwd
        [HttpPost, Route("ChangePwd")]
        public IHttpActionResult ChangePwd([FromBody] ChangePwdModel pCPwd)
        {
            if(pCPwd.newpassword != pCPwd.confirmpassword)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Please check new password and confirm new password."));
            }

            OracleCommand cmd = new OracleCommand();

            //check username & password
            cmd.CommandText = "SP_NP_CH_PWD_CHK";
            cmd.Parameters.Add("P_SK_USER_ID", OracleDbType.Varchar2).Value = pCPwd.username;
            cmd.Parameters.Add("P_PASSWORD", OracleDbType.Varchar2).Value = pCPwd.oldpassword;
            cmd.Parameters.Add("P_NEW_PASSWORD", OracleDbType.Varchar2).Value = pCPwd.newpassword;
            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
            cmd = oOracledb.SqlQueryReturnValue(cmd);

            if(cmd.Parameters["R_STATUS"].Value.ToString() == "Pass")
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_CH_PWD";
                cmd.Parameters.Add("P_SK_USER_ID", OracleDbType.Varchar2).Value = pCPwd.username;
                cmd.Parameters.Add("P_NEW_PASSWORD", OracleDbType.Varchar2).Value = pCPwd.newpassword;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
            } 
            else if (cmd.Parameters["R_STATUS"].Value.ToString() == "Fail")
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Incorrect current password."));
            } 
            else if (cmd.Parameters["R_STATUS"].Value.ToString() == "Used")
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, "Password should not same with used password."));
            }

            return Ok();
        }

        #endregion



        [HttpGet, Route("GetSystemMenu")]
        [Authorize]
        public IHttpActionResult GetSystemMenu()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_System_Menu] '{0}'";
            gSQL = String.Format(gSQL, DataService.GetIdentity(User, Cons.SS_USER_ROLE));
            //gSQL = "EXEC [sp_System_Menu] '2'";
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetFolderMenu_ByUser")]
        [Authorize]
        public IHttpActionResult GetFolderMenu_ByUser()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_System_Menu_By_User] '{0}'";
            gSQL = String.Format(gSQL, DataService.GetIdentity(User, Cons.SS_USER_NAME));
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetSystemMenuDetail")]
        [Authorize]
        public IHttpActionResult GetSystemMenuDetail()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_System_MenuDetail_List] '{0}'";
            gSQL = String.Format(gSQL, DataService.GetIdentity(User, Cons.SS_USER_ROLE));
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveFolderHeaderOrder")]
        [Authorize]
        public IHttpActionResult SaveFolderHeaderOrder([FromBody] dynamic[] FHeaders)
        {
            int iCount = 0;

            foreach (var header in FHeaders)
            {
                iCount++;
                gSQL = String.Format("EXEC [sp_FolderHeader_Update_Order] {0},{1}"
                                    , header["ID"]
                                    , iCount);
                odb.SqlExecute(gSQL, mDBName);

            }

            return Json("Done");
        }


        [HttpPost, Route("SaveFolderHeader")]
        [Authorize]
        public IHttpActionResult SaveFolderHeader([FromBody] dynamic FHeader)
        {
            var identity = User.Identity as ClaimsIdentity;
            string user = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_NAME).Value;
            string action = identity.Claims.FirstOrDefault(a => a.Type == Cons.SS_USER_ID).Value;
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_FolderHeader_Add] {0},{1},'{2}','{3}','{4}','{5}'";
            gSQL = String.Format(gSQL
                                , FHeader["ID"]
                                , FHeader["FOLDER_NO"]
                                , FHeader["FOLDER_NAME"]
                                , FHeader["FOLDER_STATUS"]
                                , user
                                , action);

            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("DeleteFolder/{folderID}")]
        [Authorize]
        public IHttpActionResult DeleteFolder(string folderID)
        {

            //Clear folder
            gSQL = String.Format("EXEC [sp_FolderDetail_Clear] {0}"
                                , folderID);
            odb.SqlExecute(gSQL, mDBName);

            //Delete folder
            gSQL = String.Format("EXEC [sp_FolderHeader_Delete] {0}"
                                , folderID);
            odb.SqlExecute(gSQL, mDBName);

            return Json("Done");
        }

        [HttpGet, Route("GetFolderHeader_ByID/{id}")]
        [Authorize]
        public IHttpActionResult GetFolderHeader_ByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_FolderHeader_List] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetFolderDetail_ByID/{folder_id}")]
        [Authorize]
        public IHttpActionResult GetFolderDetail_ByID(string folder_id)
        {
            var identity = User.Identity as ClaimsIdentity;
            string user = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_NAME).Value;
            DataTable dt = new DataTable();
            gSQL = "EXEC [sp_FolderDetail_List] '{0}',{1}";
            gSQL = String.Format(gSQL, user
                                     , folder_id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveFolderDetail/{folderID}")]
        [Authorize]
        public IHttpActionResult SaveFolderDetail(string folderID, [FromBody] dynamic[] FDetail)
        {
            DataTable dt = new DataTable();
            int iCount = 0;

            //Clear folder
            gSQL = String.Format("EXEC [sp_FolderDetail_Clear] {0}"
                                , folderID);
            odb.SqlExecute(gSQL, mDBName);

            foreach (var detail in FDetail)
            {
                iCount++;
                gSQL = "EXEC [sp_FolderDetail_Add] {0},{1},{2}";
                gSQL = String.Format(gSQL
                                    , folderID
                                    , detail["MENU_DETAIL_ID"]
                                    , iCount);

                odb.SqlExecute(gSQL, mDBName);
            }

            return Json("Done");
        }

        [HttpGet, Route("DeleteFolderDetail/{folderID}/{menuDetailID}")]
        [Authorize]
        public IHttpActionResult DeleteFolderDetail(string folderID, string menuDetailID)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_System_MenuDetail_Delete] {0},{1}";
            gSQL = String.Format(gSQL
                                , folderID
                                , menuDetailID);

            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetHeaderGridView/{tableName}")]
        [Authorize]
        public IHttpActionResult GetHeaderGridView(string tableName)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_CONF_Grid_List] '{0}','{1}' ";
            gSQL = String.Format(gSQL, tableName, DataService.GetIdentity(User, Cons.SS_USER_LANG));
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetHeaderGridViewMonitoring/{tableName}")]
        //[Authorize]
        public IHttpActionResult GetHeaderGridViewMonitoring(string tableName)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_CONF_Grid_List] '{0}','{1}' ";
            gSQL = String.Format(gSQL, tableName, DataService.GetIdentity(User, Cons.SS_USER_LANG));
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }
        [HttpPost, Route("GetHeaderGridViewMonitor")]
        //[Authorize]
        public IHttpActionResult GetHeaderGridViewMonitor([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            SqlCommand cmd = new SqlCommand();
            cmd.CommandText = "sp_CONF_Grid_Group_List";
            cmd.Parameters.Add("@TableName1", SqlDbType.NVarChar).Value = param.BY_CUST;
            cmd.Parameters.Add("@TableName2", SqlDbType.NVarChar).Value = param.BY_PICK;
            cmd.Parameters.Add("@TableName3", SqlDbType.NVarChar).Value = "";
            cmd.Parameters.Add("@TableName4", SqlDbType.NVarChar).Value = "";
            cmd.Parameters.Add("@TableName5", SqlDbType.NVarChar).Value = "";
            cmd.Parameters.Add("@Language", SqlDbType.NVarChar).Value = "Admin";//DataService.GetIdentity(User, Cons.SS_USER_LANG);
            cmd.CommandType = CommandType.StoredProcedure;

            //gSQL = "EXEC [sp_CONF_Grid_List_Monitor] {0},'{1}' ";
            //gSQL = String.Format(gSQL, tableName, DataService.GetIdentity(User, Cons.SS_USER_LANG));
            ds = odb.SqlQueryDS(cmd, mDBName);

            return Json(DTFM.convertToList(ds));
        }
        [HttpPost, Route("ReportingServiceLog")]
        [Authorize]
        public IHttpActionResult ReportingServiceLog([FromBody] dynamic param)
        {
            var identity = User.Identity as ClaimsIdentity;
            string user = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_NAME).Value;
            DataTable dt = new DataTable();
            gSQL = "EXEC [sp_Reporting_Service_log] '{0}','{1}'";
            gSQL = String.Format(gSQL, param["REPORT_NAME"]
                                     , user);
            dt = odb.SqlQuery(gSQL, mDBName);
            return Json(DTFM.convertToList(dt));
        }
        #region UserList

        [HttpGet, Route("GetUsers")]
        [Authorize]
        public IHttpActionResult GetUsers()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_User_List]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetUserByID/{id}")]
        [Authorize]
        public IHttpActionResult GetUserByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_User_List_ByID] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetUserSubByID/{id}")]
        [Authorize]
        public IHttpActionResult GetUserSubByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_User_Sub_List_ByID] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveUser")]
        [Authorize]
        public IHttpActionResult SaveUser([FromBody] dynamic userDetail)
        {
            DataTable dt = new DataTable();

            // if (!ismoUtil.CheckSpecialCharacter(userDetail["USERNAME"])) {
            //     return Json(new {Result="Username can only contains 0-9, A-Z and a-z."});
            // } else 
            if (userDetail["PASSWORD"] != userDetail["PASSWORD2"])
            {
                return Json(new { Result = "Password not matched" });
            }
            else if (userDetail["PASSWORD"] != null && userDetail["USER_TYPE"] != "Dealer" && !ismoUtil.CheckStrength(userDetail["PASSWORD"]))
            {
                return Json(new { Result = "Password must contain at least 8 characters including a-z, A-Z, 0-9 and special character !,@,#,$,%,^,&,*,?,_,~,-,£,(,)" });
            }
            else if (userDetail["ID"] == "0" && userDetail["PASSWORD"] == "" && userDetail["USER_TYPE"] != "Dealer")
            {
                return Json(new { Result = "Please fill password" });
            }
            else
            {
                gSQL = "EXEC [sp_User_Save] '{0}','{1}','{2}','{3}','{4}','{5}','{6}','{7}','{8}','{9}','{10}','{11}','{12}','{13}','{14}','{15}','{16}'";
                gSQL = String.Format(gSQL
                                    , userDetail["ID"]
                                    , userDetail["USER_TYPE"]
                                    , userDetail["TITLE_TH"]
                                    , userDetail["FIRST_NAME_TH"]
                                    , userDetail["LAST_NAME_TH"]
                                    , userDetail["TITLE_EN"]
                                    , userDetail["FIRST_NAME_EN"]
                                    , userDetail["LAST_NAME_EN"]
                                    , userDetail["REFERENCE_NO"]
                                    , userDetail["DEPARTMENT_ID"]
                                    , userDetail["POSITION_ID"]
                                    , userDetail["USER_NAME"]
                                    , userDetail["PASSWORD"]
                                    , userDetail["ROLE_ID"]
                                    , userDetail["USER_STATUS"]
                                    , userDetail["EMAIL"]
                                    , DataService.GetIdentity(User, Cons.SS_USER_ID));
                dt = odb.SqlQuery(gSQL, mDBName);

                string id = dt.Rows[0]["ID"].ToString();

                return Json(new { Result = "Done", ID = id });
            }
        }
        #endregion

        #region Department
        [HttpGet, Route("GetDepartment")]
        [Authorize]
        public IHttpActionResult GetDepartment()
        {

            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Department_List]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }
        #endregion

        #region Position
        [HttpGet, Route("GetPosition")]
        [Authorize]
        public IHttpActionResult GetPosition()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Position_List]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }
        #endregion

        #region Dealer
        [HttpGet, Route("GetOutlet")]
        [Authorize]
        public IHttpActionResult GetOutlet()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_STD_Outlet_List] ";
            dt = odb.SqlQuery(gSQL, "dbCIMS");

            return Json(DTFM.convertToList(dt));
        }
        #endregion

        #region menu

        [HttpGet, Route("GetMenuHeader")]
        [Authorize]
        public IHttpActionResult GetMenuHeader()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Header_List]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetMenuDetail")]
        [Authorize]
        public IHttpActionResult GetMenuDetail()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Detail_List]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetMenuHeaderByID/{id}")]
        [Authorize]
        public IHttpActionResult GetMenuHeaderByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Header_List_ByID] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetMenuDetailByID/{id}")]
        [Authorize]
        public IHttpActionResult GetMenuDetailByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Detail_List_ByID] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveMenuHeader")]
        [Authorize]
        public IHttpActionResult SaveMenuHeader([FromBody] dynamic MHearder)
        {
            var identity = User.Identity as ClaimsIdentity;
            string user = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_ID).Value;
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Header_Save] '{0}','{1}','{2}','{3}','{4}','{5}','{6}'";
            gSQL = String.Format(gSQL
                                , MHearder["ID"]
                                , MHearder["TITLE_NO"]
                                , MHearder["TITLE_NAME_TH"]
                                , MHearder["TITLE_NAME_EN"]
                                , MHearder["TITLE_DESC"]
                                , MHearder["TITLE_STATUS"]
                                , user);

            dt = odb.SqlQuery(gSQL, mDBName);

            return Ok("Done");
        }

        [HttpPost, Route("SaveMenuDetail")]
        [Authorize]
        public IHttpActionResult SaveMenuDetail([FromBody] dynamic MDetail)
        {
            var identity = User.Identity as ClaimsIdentity;
            string user = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_ID).Value;
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Menu_Detail_Save] '{0}','{1}','{2}','{3}','{4}','{5}','{6}','{7}','{8}','{9}','{10}','{11}'";
            gSQL = String.Format(gSQL
                                , MDetail["ID"]
                                , MDetail["HEADER_ID"]
                                , MDetail["MENU_NO"]
                                , MDetail["MENU_NAME_TH"]
                                , MDetail["MENU_NAME_EN"]
                                , MDetail["MENU_DESC"]
                                , MDetail["ROUTE_TYPE"]
                                , MDetail["ROUTE_CONTROLLER_NAME"]
                                , MDetail["ROUTE_VIEW_NAME"]
                                , MDetail["ROUTE_ADDITIONAL_ID"]
                                , MDetail["MENU_STATUS"]
                                , user);

            dt = odb.SqlQuery(gSQL, mDBName);

            return Ok("Done");
        }

        #endregion

        #region report

        [HttpGet, Route("GetReportConfigure")]
        [Authorize]
        public IHttpActionResult GetReportConfigure()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Report_Configure]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetReportConfigureByID/{id}")]
        [Authorize]
        public IHttpActionResult GetReportConfigureByID(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Report_Configure_ByID] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveReportConfigure")]
        [Authorize]
        public IHttpActionResult SaveReportConfigure([FromBody] dynamic fromdata)
        {
            DataTable dt = new DataTable();

            var identity = User.Identity as ClaimsIdentity;
            string userId = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_ID).Value;

            gSQL = "EXEC [sp_Report_Configure_Save] '{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}', '{7}', '{8}', '{9}', '{10}', '{11}', '{12}', '{13}', '{14}'";
            gSQL = String.Format(gSQL
                                , fromdata["ID"]
                                , fromdata["ReportCode"] //1
                                , fromdata["ReportName"]
                                , fromdata["ReportGroup"] //3
                                , fromdata["ParameterCount"]
                                , fromdata["ParameterName1"] //5
                                , fromdata["ParameterName2"]
                                , fromdata["ParameterName3"] //7
                                , fromdata["ParameterName4"]
                                , fromdata["ParameterName5"] //9
                                , fromdata["ParameterName6"]
                                , fromdata["ParameterName7"] //11
                                , fromdata["ParameterName9"]
                                , fromdata["ParameterName10"] //13
                                , fromdata["[Remarks]"]

                                , userId);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Ok("Done");
        }

        #endregion

        #region GridView
        [HttpGet, Route("GetGridViewHeader"), Route("GetGridViewHeader/{id}")]
        [Authorize]
        public IHttpActionResult GetGridViewHeader(string id = null)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_GridView_Header] {0}";
            gSQL = String.Format(gSQL, id != null ? "'" + id + "'" : "null");
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetGridViewDetail/{id}")]
        [Authorize]
        public IHttpActionResult GetGridViewDetail(string id)
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_GridView_Detail] '{0}'";
            gSQL = String.Format(gSQL, id);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("SaveGridView")]
        [Authorize]
        public IHttpActionResult SaveGridView([FromBody] dynamic gridDetail)
        {
            gSQL = "EXEC [sp_GridView_Update] {0},'{1}','{2}','{3}','{4}','{5}','{6}','{7}','{8}','{9}','{10}','{11}','{12}'";
            gSQL = String.Format(gSQL
                                , gridDetail["ID"]
                                , gridDetail["TABLE_NAME"]
                                , gridDetail["FIELD_NAME"]
                                , gridDetail["RNO"]
                                , gridDetail["TITLE_TH"]
                                , gridDetail["TITLE_EN"]
                                , gridDetail["WIDTH"]
                                , gridDetail["ALIGN"]
                                , gridDetail["V_ALIGN"]
                                , gridDetail["DATA_TYPE"]
                                , gridDetail["PAGE_URL"]
                                , gridDetail["LINK_FN"]
                                , gridDetail["VISIBILITY"]);
            odb.SqlExecute(gSQL, mDBName);

            return Ok("Done");
        }
        #endregion



        #region UploadFiles
        [HttpPost, Route("UploadFiles")]
        [Authorize]
        public async Task<List<UploadFileResult>> UploadFiles(HttpRequestMessage request, string FileName)
        {

            //var json = request.Content.ReadAsStringAsync().Result;
            //var jArray = JArray.Parse(json);

            string baseDir = AppDomain.CurrentDomain.BaseDirectory;
            string assetDir = ConfigurationManager.AppSettings["AssetRoot"];
            if (String.IsNullOrEmpty(assetDir))
            {
                assetDir = "UploadSys";
            }


            string rootPath = baseDir + assetDir;
            if (!Directory.Exists(rootPath))
            {
                Directory.CreateDirectory(rootPath);
            }


            var reqParts = await request.Content.ReadAsMultipartAsync();

            var uploadFiles = new List<UploadFileResult>();
            for (int i = 0; i < reqParts.Contents.Count; i++)
            {
                var content = reqParts.Contents[i];
                string OriginalFileName = content.Headers.ContentDisposition.FileName.Replace("\"", "");
                string name = Path.GetFileNameWithoutExtension(FileName);
                string ext = Path.GetExtension(FileName);
                var stream = await content.ReadAsStreamAsync();

                // Use if need to generate file name in code behide
                //string strDateTime = DateTime.Now.ToString("yyyyMMddHHmmssfff");
                //string strGuid = Guid.NewGuid().ToString().Replace("-", "").ToLower();
                //string strName = System.Text.RegularExpressions.Regex.Replace(name, "[^0-9A-Za-z๐-๙ก-ฮ]+", "-").Trim('-');
                string newFileName = $"{name}{ext}";

                //string newFileName = $"{strDateTime}-{strGuid}-{OriginalFileName}";

                using (var fileStream = new FileStream($"{rootPath}/{newFileName}", FileMode.CreateNew))
                {
                    await stream.CopyToAsync(fileStream);
                }

                uploadFiles.Add(new UploadFileResult()
                {
                    FILE_NAME = OriginalFileName,
                    FILE_LOCATION = $"{assetDir}/{newFileName}",
                });
            }

            return uploadFiles;
            //return Request.CreateResponse(HttpStatusCode.OK, JsonConvert.SerializeObject(uploadFiles));
        }

        [HttpPost, Route("UploadFilesWithName")]
        [Authorize]
        public async Task<bool> UploadFilesWithName(HttpRequestMessage request, string FileName, string Floder)
        {

            var Success = false;
            //string baseDir = AppDomain.CurrentDomain.BaseDirectory;
            string gLocation = "~/tempFile/" + Floder + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            //string assetDir = ConfigurationManager.AppSettings["AssetRoot"];
            //if (String.IsNullOrEmpty(assetDir))
            //{
            //    assetDir = "UploadSys";
            //}

            //string rootPath = baseDir + assetDir;
            if (!Directory.Exists(gWriteLocation))
            {
                Directory.CreateDirectory(gWriteLocation);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                for (int i = 0; i < reqParts.Contents.Count; i++)
                {
                    var content = reqParts.Contents[i];
                    string OriginalFileName = content.Headers.ContentDisposition.FileName.Replace("\"", "");
                    string name = Path.GetFileNameWithoutExtension(FileName);
                    string ext = Path.GetExtension(FileName);
                    var stream = await content.ReadAsStreamAsync();

                    // Use if need to generate file name in code behide
                    //string strDateTime = DateTime.Now.ToString("yyyyMMddHHmmssfff");
                    //string strGuid = Guid.NewGuid().ToString().Replace("-", "").ToLower();
                    //string strName = System.Text.RegularExpressions.Regex.Replace(name, "[^0-9A-Za-z๐-๙ก-ฮ]+", "-").Trim('-');
                    string newFileName = $"{name}{ext}";

                    //string newFileName = $"{strDateTime}-{strGuid}-{OriginalFileName}";

                    using (var fileStream = new FileStream($"{gWriteLocation}/{newFileName}", FileMode.CreateNew))
                    {
                        await stream.CopyToAsync(fileStream);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            return Success;
        }

        [HttpPost, Route("UploadFileData/{dataMonth}")]
        [Authorize]
        public async Task<HttpResponseMessage> UploadFileData(string dataMonth, HttpRequestMessage File)
        {

            try
            {
                var results = new List<UploadFileResult>();
                //results = await UploadFiles(File);
                //string result = json.Content.ReadAsStringAsync().Result;
                //result = result.Replace("\\\"", "\"");
                //result = result.Substring(1).Substring(0, result.Length - 2);
                //var uploadList = JsonConvert.DeserializeObject<List<UploadFileResult>>(json);

                gSQL = String.Format("EXEC [dbo].[sp_Import_File_Data] {0},'{1}'"
                                    , dataMonth
                                    , Createpath(results.FirstOrDefault().FILE_LOCATION));

                odb.SqlExecute(gSQL, mDBName);

                return Request.CreateResponse(HttpStatusCode.OK, JsonConvert.SerializeObject(results));
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex);
            }
        }

        [HttpPost, Route("UploadFilePath")]
        [Authorize]
        public async Task<HttpResponseMessage> UploadFilePath(string dataMonth, HttpRequestMessage File)
        {

            try
            {
                var results = new List<UploadFileResult>();
                //results = await UploadFiles(File);

                string FILE_NAME = results.FirstOrDefault().FILE_NAME.ToString();
                string FILE_LOCATION = results.FirstOrDefault().FILE_LOCATION.ToString();


                gSQL = String.Format("EXEC [dbo].[sp_Import_File_Path] '{0}','{1}'"
                                    , results.FirstOrDefault().FILE_NAME.ToString()
                                    , results.FirstOrDefault().FILE_LOCATION.ToString());

                odb.SqlExecute(gSQL, mDBName);

                return Request.CreateResponse(HttpStatusCode.OK, JsonConvert.SerializeObject(results));
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex);
            }
        }


        public static string Createpath(string filePath)
        {
            string NewPath;
            string baseDir = AppDomain.CurrentDomain.BaseDirectory;

            NewPath = baseDir + filePath.Replace("/", "\\").Replace("'", "''");

            return NewPath;
        }

        #endregion

        #region download


        //[HttpPost]
        //[AcceptVerbs("GET", "POST")]
        //[Route("Download")]
        public HttpResponseMessage downloadFile(string FILE_LOCATION, string FILE_NAME)
        {
            string path = HttpContext.Current.Server.MapPath("~/" + FILE_LOCATION);

            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }

            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            try
            {

                string mimeType = MimeMapping.GetMimeMapping(FILE_NAME);
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = FILE_NAME;
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }


            return response;
        }

        public HttpResponseMessage downloadFileFromServer(string FILE_LOCATION, string FILE_NAME)
        {
            //string path = HttpContext.Current.Server.MapPath("~/" + FILE_LOCATION);

            if (!File.Exists(FILE_LOCATION))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }

            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            try
            {

                string mimeType = MimeMapping.GetMimeMapping(FILE_NAME);
                StreamContent x = new StreamContent(new FileStream(FILE_LOCATION, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = FILE_NAME;
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }


            return response;
        }

        [HttpGet, Route("Getfilepath")]
        [Authorize]
        public IHttpActionResult Getfilepath()
        {
            DataTable dt = new DataTable();

            gSQL = "select * from TBL_SYS_Upload_Filepath";
            //gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }

        [HttpPost, Route("DownloadFileFromList")]
        public HttpResponseMessage DownloadFileFromList(List<FileDownloadViewModel> model)
        {
            var File = DownloadMultipleFilesAsZip(model);

            return File;
        }


        public HttpResponseMessage DownloadMultipleFilesAsZip(List<FileDownloadViewModel> vmodel)
        {
            if (vmodel == null)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest);
            }

            string folderPath = HttpContext.Current.Server.MapPath("~/");
            if (!Directory.Exists(folderPath))
            {
                //error
            }

            List<FileInfo> fileList = new List<FileInfo>();
            foreach (var item in vmodel)
            {
                string fullPath = folderPath + item.file_path + item.file_name;
                if (!File.Exists(fullPath))
                {
                    //Error
                }
                else
                {
                    FileInfo fileInfo = new FileInfo(fullPath);
                    fileList.Add(fileInfo);
                    item.file_path = fullPath;
                }
            }

            try
            {
                using (var memoryStream = new MemoryStream())
                {
                    using (var archive = new ZipArchive(memoryStream, ZipArchiveMode.Create, true))
                    {
                        foreach (var item in vmodel)
                        {
                            if (!File.Exists(item.file_path))
                            {
                                continue;
                            }
                            //string fileNmae = Path.GetFileNameWithoutExtension(item.FILE_NAME_CUT);
                            //string newName = string.Format("{0}", fileName );

                            //string fileNameOriginal = Path.GetFileNameWithoutExtension(item.file_name);
                            string newName = string.Format("{0}", item.file_name);
                            archive.CreateEntryFromFile(item.file_path, newName, CompressionLevel.Optimal);

                        }
                    }

                    HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                    response.Content = new ByteArrayContent(memoryStream.ToArray());
                    //response.Content = new StreamContent(memoryStream);
                    response.Content.Headers.ContentLength = memoryStream.Length;
                    response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/zip");
                    response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                    response.Content.Headers.ContentDisposition.FileName = "Attachments.zip";
                    return response;
                }
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }

        public DataTable ReadDataExcel(string filepath)
        {
            System.Text.Encoding.RegisterProvider(System.Text.CodePagesEncodingProvider.Instance);

            using (FileStream stream = File.Open(filepath, FileMode.Open, FileAccess.Read))
            {
                IExcelDataReader excelReader = ExcelReaderFactory.CreateOpenXmlReader(stream);
                //excelReader.IsFirstRowAsColumnNames  = true;
                //DataSet result = excelReader.AsDataSet();

                DataSet result = excelReader.AsDataSet(new ExcelDataSetConfiguration()
                {
                    ConfigureDataTable = (_) => new ExcelDataTableConfiguration()
                    {
                        UseHeaderRow = true
                    }
                });
                DataTable dt = new DataTable();
                dt = result.Tables[0];
                return dt;
            }
        }

        public DataTable ReadDataExcel(MemoryStream stream)
        {
            IExcelDataReader excelReader = ExcelReaderFactory.CreateOpenXmlReader(stream);

            DataSet result = excelReader.AsDataSet(new ExcelDataSetConfiguration()
            {
                ConfigureDataTable = (_) => new ExcelDataTableConfiguration()
                {
                    UseHeaderRow = true
                }
            });
            DataTable dt = new DataTable();
            dt = result.Tables[0];
            return dt;
        }
        
        public class fileViewModel
        {
            public string FILE_LOCATION { get; set; }
            public string FILE_NAME { get; set; }
        }


        #endregion


        [HttpGet, Route("GetFormName/{FORM_ID}")]
        //[Authorize]
        public IHttpActionResult GetFormName(string FORM_ID)
        {
            DataTable dt = new DataTable();
            string gMsg;

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "sp_NP_MT_FORM_NAME";
            cmd.Parameters.Add("P_SK_ID", OracleDbType.Varchar2, FORM_ID, ParameterDirection.Input);
            cmd.Parameters.Add("C_SK_TNAME", OracleDbType.Varchar2, ParameterDirection.Output);
            cmd.Parameters.Add("C_SK_ENAME", OracleDbType.Varchar2, ParameterDirection.Output);
            cmd.Parameters["C_SK_TNAME"].Size = 300;
            cmd.Parameters["C_SK_ENAME"].Size = 300;

            cmd = oOracledb.SqlQueryReturnValue(cmd);

            if (DataService.GetIdentity(User, Cons.SS_USER_LANG) == "TH")
            {
                gMsg = cmd.Parameters["C_SK_TNAME"].Value.ToString();
            }
            else
            {
                gMsg = cmd.Parameters["C_SK_ENAME"].Value.ToString();
            }

            return Json(new { FORM_NAME = gMsg });
        }


        [HttpGet, Route("FindPO/{PO_NO}")]
        [Authorize]
        public IHttpActionResult FindPO(string PO_NO)
        {
            DataSet ds = new DataSet();

            PO_NO = ismoUtil.escapeCharacter(PO_NO);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_PO_FIND";
            cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2, PO_NO, ParameterDirection.Input);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));

        }


        [HttpGet, Route("SearchPO/{PO_NO}/{PO_DATE}/{INV_NO}/{INV_DATE}")]
        [Authorize]
        public IHttpActionResult SearchPO(string PO_NO, string PO_DATE, string INV_NO, string INV_DATE)
        {
            DataSet ds = new DataSet();

            PO_NO = ismoUtil.escapeCharacter(PO_NO);
            INV_NO = ismoUtil.escapeCharacter(INV_NO);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_PO_SEARCH";
            cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2, PO_NO, ParameterDirection.Input);
            cmd.Parameters.Add("P_PO_DATE", OracleDbType.Varchar2, PO_DATE, ParameterDirection.Input);
            cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2, INV_NO, ParameterDirection.Input);
            cmd.Parameters.Add("P_INV_DATE", OracleDbType.Varchar2, INV_DATE, ParameterDirection.Input);

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));

        }

        [HttpGet, Route("QueryData/{spName}")]
        [Authorize]
        public IHttpActionResult QueryData(string spName)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = spName;
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("GetTariffCode/{filter}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetTariffCode(string filter, string noofshowrec)
        {
            DataSet ds = new DataSet();

            filter = ismoUtil.escapeCharacter(filter);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_TARIFF_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
            cmd.Parameters.Add("P_NAME_FILTER", OracleDbType.Varchar2, filter, ParameterDirection.Input);
            //cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16, noofshowrec, ParameterDirection.Input);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("GetFormUserRole/{formId}")]
        [Authorize]
        public IHttpActionResult GetFormUserRole(string formId)
        {
            string userName = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            string roleId = DataService.GetIdentity(User, Cons.SS_USER_ROLE);

            //Check exists in MSSQL
            gSQL = "EXEC [sp_System_Menu] '{0}'";
            gSQL = String.Format(gSQL, roleId);
            DataTable menuDT = odb.SqlQuery(gSQL, mDBName);

            if(menuDT.Select(String.Format("ROUTE_VIEW_NAME = '{0}'", formId)).Length > 0){
                odb.WriteMenuLog(formId, userName);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_FORM_ROLE_BYUSER";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = userName;
                cmd.Parameters.Add("P_FORM_ID", OracleDbType.Varchar2).Value = formId;
                cmd.Parameters.Add("R_INS", OracleDbType.Varchar2, 5).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CHG", OracleDbType.Varchar2, 5).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DEL", OracleDbType.Varchar2, 5).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INQ", OracleDbType.Varchar2, 5).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                var returnObj = new
                {
                    STATUS = "Done",
                    INS = cmd.Parameters["R_INS"].Value.ToString(),
                    CHG = cmd.Parameters["R_CHG"].Value.ToString(),
                    DEL = cmd.Parameters["R_DEL"].Value.ToString(),
                    INQ = cmd.Parameters["R_INQ"].Value.ToString()
                };

                return Ok(returnObj);
            } else
            {
                return BadRequest();
            }

        }


        #region MA:system tracking

        [HttpGet, Route("GetCurrentDate")]
        [Authorize]
        public IHttpActionResult GetCurrentDate()
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_CURRENT_DATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("C_CURRENT_DATE", OracleDbType.Varchar2, 25).Direction = ParameterDirection.Output;

            cmd = oOracledb.SqlQueryReturnValue(cmd);

            return Ok(new
            {
                currentDate = DateTime.Parse(cmd.Parameters["C_CURRENT_DATE"].Value.ToString())
            });
        }

        [HttpGet, Route("GetBatchStatus")]
        [Authorize]
        public IHttpActionResult GetBatchStatus()
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_MT_BATCH_CONTROL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Ok(DTFM.convertToList(ds));
        }

        [HttpGet, Route("GetSystemLogs")]
        [Authorize]
        public IHttpActionResult GetSystemLogsWeb()
        {
            DataTable dt = GetSystemLogs(0);

            return Ok(DTFM.convertToList(dt));
        }

        [HttpGet, Route("GetSystemLogs/batch/{duration}")]
        [Authorize]
        public IHttpActionResult GetSystemLogsBatch(int duration)
        {
            try
            {
                if(DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_SystemCheckup")
                {
                    throw new Exception("Invalid");
                }

                List<string> messageList;
                String DateFormat = DateTime.Now.ToString("yyyyMMddhHHmmss");
                List<EmailAttachment> attachments = new List<EmailAttachment>();

                /* Sql Server */
                DataTable dtMSSQL = GetSystemLogs(duration);
                messageList = new List<string>();
                messageList.Add($"<LOG_DATETIME>: (<LOG_TYPE>) <LOG_MESSAGE> [<REF>]\r\n");
                foreach (DataRow dr in dtMSSQL.Rows)
                {
                    messageList.Add($"{dr["LOG_DATETIME"]}: ({dr["LOG_TYPE"]}) {dr["LOG_MESSAGE"]} [{dr["REF"]}]\r\n");
                }

                byte[] bytesMSSQL = dataService.WriteBytes(messageList);
                attachments.Add(new EmailAttachment(bytesMSSQL, String.Concat("MSSQL-dbSYS-", DateFormat, ".log"), "plain/text"));

                /* Oracle DB: Track logs */
                DataTable dtOracle = GetTrackingLogs(duration);
                messageList = new List<string>();
                messageList.Add($"<LOG_DATE_TIME>: (<LOG_TYPE>) <MESSAGE> [<REMARKS>]\r\n");
                foreach (DataRow dr in dtOracle.Rows)
                {
                    messageList.Add($"{dr["LOG_DATE_TIME"]}: ({dr["LOG_TYPE"]}) {dr["MESSAGE"]} [{dr["REMARKS"]}]\r\n");
                }

                byte[] bytesOracle = dataService.WriteBytes(messageList);
                attachments.Add(new EmailAttachment(bytesOracle, String.Concat("Oracle-", DateFormat, ".log"), "plain/text"));

                /* Oracle DB: Process logs */
                DataTable dtOracle2 = GetProcessLogs(duration);
                messageList = new List<string>();
                messageList.Add($"<LOG_DATE_TIME>: (<LOG_TYPE>) <MESSAGE> [<PROGRAM_NAME>]\r\n");
                foreach (DataRow dr in dtOracle2.Rows)
                {
                    messageList.Add($"{dr["LOG_DATE_TIME"]}: ({dr["LOG_TYPE"]}) {dr["MESSAGE"]} [{dr["PROGRAM_NAME"]}]\r\n");
                }

                byte[] bytesOracle2 = dataService.WriteBytes(messageList);
                attachments.Add(new EmailAttachment(bytesOracle2, String.Concat("Process-", DateFormat, ".log"), "plain/text"));

                if (dtMSSQL.Rows.Count > 0 || dtOracle.Rows.Count > 0)
                {
                    //Get email configuration
                    EmailConfigure configure = dataService.GetEmailConfigure("SystemLogs");
                    configure.attachments = attachments.ToArray();

                    //Send email with attachments
                    EmailSender sender = new EmailSender();
                    sender.Email(configure);
                }

                return Ok();
            
            } catch(Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        [HttpGet, Route("GetUserMenuLogs")]
        [Authorize]
        public IHttpActionResult GetUserMenuLogs()
        {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Get_User_Menu_Logs]";
            gSQL = String.Format(gSQL);
            dt = odb.SqlQuery(gSQL, mDBName);

            return Json(DTFM.convertToList(dt));
        }


        [HttpGet, Route("GetSystemFailure")]
        [Authorize]
        public IHttpActionResult GetSystemFailure()
        {
            try
            {
                if(DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_SystemCheckup")
                {
                    throw new Exception("Invalid");
                }

                List<string> messageList;
                String DateFormat = DateTime.Now.ToString("yyyyMMddhHHmmss");
                List<EmailAttachment> attachments = new List<EmailAttachment>();

                /*Lock sessions*/
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = @"SELECT LCK.*, OBJ.OBJECT_NAME, SES.TERMINAL
                                      FROM V$LOCK LCK, ALL_OBJECTS OBJ, V$SESSION SES
                                    WHERE LCK.SID > 7
                                       AND LCK.LMODE IN ('2','3')
                                       AND OBJ.OWNER = 'DEV'
                                       AND OBJ.OBJECT_TYPE = 'TABLE'
                                       AND LCK.ID1 = OBJ.OBJECT_ID
                                       AND LCK.SID = SES.SID";
                DataTable tableLockSession = oOracledb.SqlQueryText(cmd);

                messageList = new List<string>();
                messageList.Add($"<SID>|<TERMINAL> \"<OBJECT_NAME>\" (<TYPE>)<ADDR>,<KADDR> [<ID1>][<ID2>][<LMODE>][<REQUEST>][<CTIME>][<BLOCK>]\r\n");
                foreach (DataRow dr in tableLockSession.Rows)
                {
                    messageList.Add($"{dr["SID"]}|{dr["TERMINAL"]} \"{dr["OBJECT_NAME"]}\" ({dr["TYPE"]}){dr["ADDR"]},{dr["KADDR"]} [{dr["ID1"]}][{dr["ID2"]}][{dr["LMODE"]}][{dr["REQUEST"]}][{dr["CTIME"]}][{dr["BLOCK"]}]\r\n");
                }

                byte[] bytesLock = dataService.WriteBytes(messageList);
                attachments.Add(new EmailAttachment(bytesLock, String.Concat("session-lock-", DateFormat, ".log"), "plain/text"));


                /*Waiting sessions*/
                cmd = new OracleCommand();
                cmd.CommandText = @"SELECT VH.SID       LOCKING_SID,
                                           VS.TERMINAL  LOCKING_TERMINAL,
                                           VS.STATUS    STATUS,
                                           VS.PROGRAM   PROGRAM_HOLDING,
                                           VW.SID       WAITER_SID,
                                           VSW.TERMINAL WAITER_TERMINAL,
                                           VSW.PROGRAM  PROGRAM_WAITING
                                      FROM V$LOCK VH, V$LOCK VW, V$SESSION VS, V$SESSION VSW
                                    WHERE (VH.ID1, VH.ID2) IN
                                           (SELECT ID1, ID2
                                              FROM V$LOCK
                                             WHERE REQUEST = 0
                                            INTERSECT
                                            SELECT ID1, ID2 FROM V$LOCK WHERE LMODE = 0)
                                       AND VH.ID1 = VW.ID1
                                       AND VH.ID2 = VW.ID2
                                       AND VH.REQUEST = 0
                                       AND VW.LMODE = 0
                                       AND VH.SID = VS.SID
                                       AND VW.SID = VSW.SID";
                DataTable tableWaitSession = oOracledb.SqlQueryText(cmd);
                messageList = new List<string>();
                messageList.Add($"LOCK: <LOCKING_SID>|<LOCKING_TERMINAL> <PROGRAM_HOLDING> (<STATUS>) --- WAIT: <WAITER_SID>|<WAITER_TERMINAL> (<STATUS>) <PROGRAM_WAITING>\r\n");
                foreach (DataRow dr in tableWaitSession.Rows)
                {
                    messageList.Add($"LOCK: {dr["LOCKING_SID"]}|{dr["LOCKING_TERMINAL"]} {dr["PROGRAM_HOLDING"]} ({dr["STATUS"]}) --- WAIT: {dr["WAITER_SID"]}|{dr["WAITER_TERMINAL"]} ({dr["STATUS"]}) {dr["PROGRAM_WAITING"]}\r\n");
                }

                byte[] bytesWait = dataService.WriteBytes(messageList);
                attachments.Add(new EmailAttachment(bytesWait, String.Concat("session-wait-", DateFormat, ".log"), "plain/text"));

                if (tableLockSession.Rows.Count > 0 || tableWaitSession.Rows.Count > 0)
                {
                    //Get email configuration
                    EmailConfigure configure = dataService.GetEmailConfigure("SystemFailure");
                    configure.attachments = attachments.ToArray();

                    //Send email with attachments
                    EmailSender sender = new EmailSender();
                    sender.Email(configure);
                }

                return Ok();

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        [HttpPost, Route("SendEmail/{emailType}")]
        public IHttpActionResult SendEmailRequest(string emailType, [FromBody] dynamic fileInfo)
        {
            try
            {
                List<EmailAttachment> attachments = new List<EmailAttachment>();

                //Get email configuration
                EmailConfigure configure = dataService.GetEmailConfigure(emailType);

                //Add file attachment
                if (fileInfo != null)
                {
                    string filePath = "";

                    try
                    {
                        filePath = HttpContext.Current.Server.MapPath(fileInfo.path.ToString());
                    } catch (Exception ex) {
                        filePath = fileInfo.path.ToString();
                    }

                    Byte[] bytes = File.ReadAllBytes(filePath);

                    attachments.Add(new EmailAttachment(bytes, fileInfo.name.ToString(), "plain/text"));
                }

                configure.attachments = attachments.ToArray();

                //Send email with attachments
                EmailSender sender = new EmailSender();
                sender.Email(configure);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", string.Concat(emailType,": ",ex.Message), "SendEmail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        private DataTable GetSystemLogs(int duration) {
            DataTable dt = new DataTable();

            gSQL = "EXEC [sp_Get_System_Logs] '{0}'";
            gSQL = String.Format(gSQL, duration.ToString());
            dt = odb.SqlQuery(gSQL, mDBName);

            return dt;
        }

        private DataTable GetTrackingLogs(int duration)
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_GET_TRACKING_LOGS";
            cmd.Parameters.Add("P_DURATION", OracleDbType.Decimal).Value = duration;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dt = ds.Tables[0];

            return dt;
        }

        private DataTable GetProcessLogs(int duration)
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_GET_PROCESS_LOGS";
            cmd.Parameters.Add("P_DURATION", OracleDbType.Decimal).Value = duration;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dt = ds.Tables[0];

            return dt;
        }

        public DataTable GetProcessLogs(string programName, DateTime procDate)
        {

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_GET_PROCESS_LOGS_PRG";
            cmd.Parameters.Add("P_PROGRAM_NAME", OracleDbType.Varchar2).Value = programName;
            cmd.Parameters.Add("P_PROC_DATE", OracleDbType.Date).Value = procDate;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dt = ds.Tables[0];

            return dt;
        }

        #endregion

        #region MA:system user & password
        [HttpGet, Route("SKCUMUSER_GetUserID/{userID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_GetUserID(string userID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            userID = ismoUtil.escapeCharacter(userID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_M_GET_USER";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = userID.ToUpper() == "NULL" ? (object)DBNull.Value : userID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_GetUserID");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SKCUMUSER_GetSystemID/{systemID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_GetSystemID(string systemID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            systemID = ismoUtil.escapeCharacter(systemID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_M_SYSTEM_ID";
                cmd.Parameters.Add("P_SYSTEM_ID", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_GetSystemID");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SKCUMUSER_GetUserGroup/{groupID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_GetUserGroup(string groupID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            groupID = ismoUtil.escapeCharacter(groupID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_M_USERGROUP";
                cmd.Parameters.Add("P_GRP_USER_ID", OracleDbType.Varchar2).Value = groupID.ToUpper() == "NULL" ? (object)DBNull.Value : groupID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_GetUserGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SKCUMUSER_GetData")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_GetData([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_M_GET_DATA";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = pData.SK_USER_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SKCUMUSER_Save")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_Save([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_SAVE";
                cmd.Parameters.Add("P_SK_USER_ID", OracleDbType.Varchar2).Value = pData.SK_USER_ID;
                cmd.Parameters.Add("P_SK_SYSTEM_ID", OracleDbType.Varchar2).Value = pData.SK_SYSTEM_ID;
                cmd.Parameters.Add("P_SK_GRP_USER_ID", OracleDbType.Varchar2).Value = pData.SK_GRP_USER_ID;
                cmd.Parameters.Add("P_TNAME", OracleDbType.Varchar2).Value = pData.TNAME;
                cmd.Parameters.Add("P_LANGUAGE", OracleDbType.Varchar2).Value = pData.LANGUAGE;
                cmd.Parameters.Add("P_LIMIT_FLAG", OracleDbType.Varchar2).Value = pData.LIMIT_FLAG;
                cmd.Parameters.Add("P_EXPIRE_DATE", OracleDbType.Date).Value = pData.EXPIRE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.EXPIRE_DATE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.START_DATE);
                cmd.Parameters.Add("P_ACTIVE", OracleDbType.Varchar2).Value = pData.ACTIVE;
                cmd.Parameters.Add("P_SET_TOGGLE", OracleDbType.Varchar2).Value = pData.SET_TOGGLE;
                cmd.Parameters.Add("P_ENAME", OracleDbType.Varchar2).Value = pData.ENAME;
                cmd.Parameters.Add("P_EMAIL", OracleDbType.Varchar2).Value = pData.REMARK;
                cmd.Parameters.Add("P_HISTORY", OracleDbType.Varchar2).Value = pData.HISTORY;
                cmd.Parameters.Add("P_TITLE", OracleDbType.Varchar2).Value = pData.TITLE;
                cmd.Parameters.Add("P_SK_USER_PASSWORD", OracleDbType.Varchar2).Value = pData.SK_USER_PASSWORD;
                cmd.Parameters.Add("P_LOGIN_FAILURE", OracleDbType.Varchar2).Value = pData.LOGIN_FAILURE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                gSQL = "EXEC [sp_User_Add] '{0}','{1}','{2}','{3}','{4}','{5}','{6}','{7}'";
                gSQL = String.Format(gSQL
                                    , pData.SK_USER_ID
                                    , pData.FIRST_NAME_TH
                                    , pData.LAST_NAME_TH
                                    , pData.FIRST_NAME_EN
                                    , pData.LAST_NAME_EN
                                    , pData.SK_GRP_USER_ID
                                    , pData.ACTIVE
                                    , DataService.GetIdentity(User, Cons.SS_USER_ID));
                dt = odb.SqlQuery(gSQL, mDBName);
                return Json(new { STATUS = "DONE" });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SKCUMUSER_CheckUser")]
        [Authorize]
        public IHttpActionResult SKCUMUSER_CheckUser([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMUSER_M_CHK_USER";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = pData.SK_USER_ID;
                cmd.Parameters.Add("P_SYSTEM_ID", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMUSER_CheckUser");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region MA:User PRG 

        [HttpGet, Route("SKCUMTPRG_GetPRGID/{sk_prg_id}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SKCUMTPRG_GetPRGID(string sk_prg_id, int noofshowrec)
        {
            DataSet ds = new DataSet();
            sk_prg_id = ismoUtil.escapeCharacter(sk_prg_id);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMTPRG_M_GET_USER";
                cmd.Parameters.Add("P_PRG_ID", OracleDbType.Varchar2).Value = sk_prg_id.ToUpper() == "NULL" ? (object)DBNull.Value : sk_prg_id;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMTPRG_GetPRGID");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SKCUMTPRG_GetData")]
        [Authorize]
        public IHttpActionResult SKCUMTPRG_GetData([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMTPRG_M_GET_DATA";
                cmd.Parameters.Add("P_PRG_ID", OracleDbType.Varchar2).Value = pData.SK_PRG_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMTPRG_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SKCUMTPRG_CheckUser")]
        [Authorize]
        public IHttpActionResult SKCUMTPRG_CheckUser([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMTPRG_M_CHK_USER";
                cmd.Parameters.Add("P_PRG_ID", OracleDbType.Varchar2).Value = pData.SK_PRG_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMTPRG_CheckUser");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SKCUMTPRG_Save")]
        [Authorize]
        public IHttpActionResult SKCUMTPRG_Save([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SKCUMTPRG_SAVE";
                cmd.Parameters.Add("P_SK_PRG_ID", OracleDbType.Varchar2).Value = pData.SK_PRG_ID;
                cmd.Parameters.Add("P_PRG_TYPE", OracleDbType.Varchar2).Value = pData.PRG_TYPE;
                cmd.Parameters.Add("P_TNAME", OracleDbType.Varchar2).Value = pData.TNAME;
                cmd.Parameters.Add("P_ENAME", OracleDbType.Varchar2).Value = pData.ENAME;
                cmd.Parameters.Add("P_ICON", OracleDbType.Varchar2).Value = pData.ICON;
                cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = pData.REMARK;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { STATUS = "DONE" });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SKCUMTPRG_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion
        #region ReportViewer
        [HttpGet, Route("GetReportViewerList")]
        [Authorize]
        public IHttpActionResult GetReportViewerList()
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_List";
                command.Parameters.Add("@ROLE_ID", SqlDbType.Int).Value = Convert.ToInt32(DataService.GetIdentity(User, Cons.SS_USER_ROLE));

                DataSet ds = odb.SqlQueryDS(command, mDBName);
                DataTable reportList = ds.Tables[0];

                return Ok(DTFM.convertToList(reportList));

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "GetReportViewerList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        [HttpGet, Route("GetReportViewer/{reportId}")]
        [Authorize]
        public IHttpActionResult GetReportViewer(int reportId)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_ByID";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = reportId;
                command.Parameters.Add("@P_USER_ID", SqlDbType.VarChar).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                DataSet ds = odb.SqlQueryDS(command, mDBName);

                return Ok(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle","Report ID: " + reportId + " " + ex.Message, "GetReportViewer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        [HttpPost, Route("RunReportViewer")]
        [Authorize]
        public IHttpActionResult RunReportViewer([FromBody] dynamic pData)
        {

            try
            {
                string query = String.Concat("SELECT Q.* FROM (", pData.query, ") Q");

                OracleCommand command = new OracleCommand();
                command.CommandType = CommandType.Text;
                command.CommandText = query;

                DataTable dataTable = oOracledb.SqlQueryText(command);

                return Ok(DTFM.convertToList(dataTable));

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle","Report ID: " + pData.ID + " " + ex.Message, "GetReportViewer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        [HttpPost, Route("SaveReportViewer")]
        [Authorize]
        public IHttpActionResult SaveReportViewer([FromBody] dynamic pData)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_Save";
                command.Parameters.Add("@ID", SqlDbType.Int).Value = pData.ID;
                command.Parameters.Add("@HEADER_ID", SqlDbType.Int).Value = pData.HEADER_ID;
                command.Parameters.Add("@REPORT_NO", SqlDbType.Int).Value = pData.REPORT_NO;
                command.Parameters.Add("@REPORT_NAME", SqlDbType.NVarChar, 500).Value = pData.REPORT_NAME;
                command.Parameters.Add("@REP_SELECT", SqlDbType.NVarChar).Value = pData.REP_SELECT;
                command.Parameters.Add("@REP_FROM", SqlDbType.NVarChar, 2000).Value = pData.REP_FROM;
                command.Parameters.Add("@REP_WHERE", SqlDbType.NVarChar).Value = pData.REP_WHERE;
                command.Parameters.Add("@REP_GROUP_BY", SqlDbType.NVarChar, 2000).Value = pData.REP_GROUP_BY;
                command.Parameters.Add("@REP_ORDER_BY", SqlDbType.NVarChar, 2000).Value = pData.REP_ORDER_BY;
                command.Parameters.Add("@REP_GROUP", SqlDbType.NVarChar, 500).Value = pData.REP_GROUP;
                command.Parameters.Add("@IS_PARAMETER", SqlDbType.NVarChar, 10).Value = pData.IS_PARAMETER;
                command.Parameters.Add("@REMARK", SqlDbType.NVarChar, 500).Value = pData.REMARK;
                odb.SqlExecute(command, mDBName);
                return Ok("Done");
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }
        [HttpPost, Route("SaveReportViewerParams/{reportId}")]
        [Authorize]
        public IHttpActionResult SaveReportViewerParams(int reportId, [FromBody] dynamic pData)
        {
            try
            {
                var paramList = new List<dynamic>(pData.paramList);
                int count = 0;

                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_Clear_Params";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = reportId;

                odb.SqlExecute(command, mDBName);
                command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Report_Viewer_Save_Name";
                command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = reportId;
                command.Parameters.Add("@REPORT_NAME", SqlDbType.NVarChar, 500).Value = pData.reportName;

                odb.SqlExecute(command, mDBName);
                foreach (var param in paramList)
                {
                    count++;

                    command = new SqlCommand();
                    command.CommandType = CommandType.StoredProcedure;
                    command.CommandText = "dbo.sp_Report_Viewer_Save_Params";
                    command.Parameters.Add("@REPORT_ID", SqlDbType.Int).Value = reportId;
                    command.Parameters.Add("@PARAM_NO", SqlDbType.Int).Value = count;
                    command.Parameters.Add("@CONDITION", SqlDbType.NVarChar, -1).Value = param.CONDITION;
                    command.Parameters.Add("@REMARKS", SqlDbType.NVarChar, 500).Value = param.REMARKS;
                    command.Parameters.Add("@IS_ACTIVE", SqlDbType.NVarChar, 10).Value = param.IS_ACTIVE;

                    odb.SqlExecute(command, mDBName);

                }

                return Ok("Done");

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "SaveReportViewerParams");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        #endregion

        #region ManualDocument
        [HttpGet, Route("GetManualDocumentList")]
        [Authorize]
        public IHttpActionResult GetManualDocumentList()
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Manual_Document_List";
                command.Parameters.Add("@DOCUMENT_MODULE", SqlDbType.NVarChar, 50).Value = "Warehouse";
                DataSet ds = odb.SqlQueryDS(command, mDBName);
                DataTable dt = ds.Tables[0];

                return Ok(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "GetManualDocumentList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        [HttpPost, Route("GetManualDocumentByName")]
        [Authorize]
        public IHttpActionResult GetManualDocumentByName([FromBody] dynamic data)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Manual_Document_List_ByName";
                command.Parameters.Add("@DOCUMENT_MODULE", SqlDbType.NVarChar, 50).Value = "Warehouse";
                command.Parameters.Add("@DOCUMENT_NAME", SqlDbType.NVarChar, 250).Value = data["DOCUMENT_NAME"] != null ? data["DOCUMENT_NAME"].ToString().Trim() : "";                
                DataSet ds = odb.SqlQueryDS(command, mDBName);
                DataTable dt = ds.Tables[0];

                return Ok(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "GetManualDocumentByName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        } 
        
        [HttpPost, Route("ManualDocumentSave")]
        [Authorize]
        public IHttpActionResult ManualDocumentSave([FromBody] dynamic data)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Manual_Document_Save";
                command.Parameters.Add("@DOCUMENT_ID", SqlDbType.Int).Value = data["DOCUMENT_ID"] ?? 0;
                command.Parameters.Add("@DOCUMENT_MODULE", SqlDbType.NVarChar, 50).Value = data["DOCUMENT_MODULE"].ToString().Trim();
                command.Parameters.Add("@DOCUMENT_NAME", SqlDbType.NVarChar, 250).Value = data["DOCUMENT_NAME"].ToString().Trim();
                command.Parameters.Add("@DOCUMENT_PATH", SqlDbType.NVarChar, 500).Value = data["DOCUMENT_PATH"].ToString().Trim();
                odb.SqlExecute(command, mDBName);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "ManualDocumentSave");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        [HttpPost, Route("ManualDocumentDelete")]
        [Authorize]
        public IHttpActionResult ManualDocumentDelete([FromBody] dynamic data)
        {
            try
            {
                SqlCommand command = new SqlCommand();
                command.CommandType = CommandType.StoredProcedure;
                command.CommandText = "dbo.sp_Manual_Document_Delete";
                command.Parameters.Add("@DOCUMENT_ID", SqlDbType.Int).Value = data["DOCUMENT_ID"];
                odb.SqlExecute(command, mDBName);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                odb.WriteLog("ErrorHandle", ex.Message, "ManualDocumentDelete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError));
            }
        }

        #endregion
    }

    public class DownloadViewModel
    {

        public IList<FileDownloadViewModel> FILES { get; set; }
    }

    public class FileDownloadViewModel
    {
        //public string FILE_NAME_CUT { get; set; }
        public string file_name { get; set; }
        public string file_path { get; set; }
    }

    public class UploadFileResult
    {
        public string FILE_NAME { get; set; }
        public string FILE_LOCATION { get; set; }
    }

    public class ChangePwdModel
    {
        public string username { get; set; }
        public string oldpassword { get; set; }
        public string newpassword { get; set; }
        public string confirmpassword { get; set; }
    }
}

