﻿using SYS_Core.aControl;
using SYS_Core.Services;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.Cors;

namespace SYS_Core.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/token")]
    public class TokenController : ApiController
    {

        private DBManager odb = new DBManager();
        public TokenService _tokenService = new TokenService();

        [HttpPost, Route("Refresh")]
        public IHttpActionResult Refresh([FromBody] dynamic oToken)
        {
            string token = oToken["token"].ToString();
            string refreshToken = oToken["refreshToken"].ToString();

            var principal = _tokenService.GetPrincipalFromExpiredToken(token);
            var identity = principal.Identity as System.Security.Claims.ClaimsIdentity;
            string username = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_NAME).Value;

            var newJwtToken = _tokenService.GenerateAccessToken(principal.Claims);

            if(refreshToken != null)
            {
                var newRefreshToken = _tokenService.GenerateRefreshToken();

                string TokenRegisterResult = odb.RegisterToken(username, token, refreshToken, newJwtToken, newRefreshToken);

                if (TokenRegisterResult == "VALID")
                {
                    return Ok(new
                    {
                        token = newJwtToken,
                        refreshToken = newRefreshToken
                    });
                }
                else
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable));
                }

            } else //For handheld
            {
                return Ok(new { token = newJwtToken });
            }
            
        }

        [HttpPost, Route("Check")]
        public IHttpActionResult Check([FromBody] dynamic oToken)
        {
            string token = oToken["token"].ToString();
            string refreshToken = oToken["refreshToken"].ToString();

            if (token != null && token != "")
            {
                var principal = _tokenService.GetPrincipalFromExpiredToken(token);

                var identity = principal.Identity as System.Security.Claims.ClaimsIdentity;
                string username = identity.Claims.FirstOrDefault(x => x.Type == Cons.SS_USER_NAME).Value;

                var newJwtToken = _tokenService.GenerateAccessToken(principal.Claims);
                var newRefreshToken = _tokenService.GenerateRefreshToken();

                bool TokenResult = odb.CheckToken(username, token, refreshToken);

                if (TokenResult)
                {
                    return Ok();
                }
            }
            return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable));
        }

        //[HttpPost, Route("Revoke")]
        //[Authorize]
        //public async Task<IHttpActionResult> Revoke()
        //{
        //    var username = User.Identity.Name;

        //    var user = _usersDb.Users.SingleOrDefault(u => u.Username == username);
        //    if (user == null) return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));

        //    user.RefreshToken = null;

        //    await _usersDb.SaveChangesAsync();

        //    return Ok();
        //}

    }
}